#nullable enable
using CanonSharp.Benchmark.Canon.Core.GrammarParser;
using CanonSharp.Benchmark.Canon.Core.Abstractions;
using CanonSharp.Benchmark.Canon.Core.Enums;

namespace CanonSharp.Benchmark.Canon.Core.GrammarParser;

public class GeneratedTransformer : ITransformer
{
    private IDictionary<TerminatorBase, string> _shiftPointers;

    public string Name { get; }

    public IDictionary<Terminator, ReduceInformation> ReduceTable { get; }

    public IDictionary<TerminatorBase, ITransformer> ShiftTable { get; }

    public GeneratedTransformer(Dictionary<TerminatorBase, string> shiftTable,
        Dictionary<Terminator, ReduceInformation> reduceTable, string name)
    {
        ReduceTable = reduceTable;
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = shiftTable;
        Name = name;
    }

    public GeneratedTransformer()
    {
        ReduceTable = new Dictionary<Terminator, ReduceInformation>();
        ShiftTable = new Dictionary<TerminatorBase, ITransformer>();
        _shiftPointers = new Dictionary<TerminatorBase, string>();
        Name = Guid.NewGuid().ToString();
    }

    public void ConstructShiftTable(Dictionary<string, GeneratedTransformer> transformers)
    {
        foreach (KeyValuePair<TerminatorBase,string> pair in _shiftPointers)
        {
            ShiftTable.Add(pair.Key, transformers[pair.Value]);
        }
    }

    public override bool Equals(object? obj)
    {
        if (obj is not GeneratedTransformer other)
        {
            return false;
        }

        return Name == other.Name;
    }

    public override int GetHashCode() => Name.GetHashCode();
}
public class GeneratedGrammarParser : IGrammarParser
{
	private static readonly Dictionary<string, GeneratedTransformer> s_transformers = new()
	{
		{ "59d736fd-fe3f-42d7-9c76-7dab621a50ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramStruct), "6a1b6afe-030a-4f04-b984-d60ce54ee902"}, { new NonTerminator(NonTerminatorType.ProgramHead), "bcccfbd3-94af-4a1e-afcb-396a16a48dad"}, { new Terminator(KeywordType.Program), "ac8e0bb9-90c9-4833-8423-d16fde25b4e4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59d736fd-fe3f-42d7-9c76-7dab621a50ad") },
		{ "6a1b6afe-030a-4f04-b984-d60ce54ee902", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.StartNonTerminator))}, }, "6a1b6afe-030a-4f04-b984-d60ce54ee902") },
		{ "bcccfbd3-94af-4a1e-afcb-396a16a48dad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "577a0dee-e956-4d99-8fbd-97acdd8ac3df"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bcccfbd3-94af-4a1e-afcb-396a16a48dad") },
		{ "ac8e0bb9-90c9-4833-8423-d16fde25b4e4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "59d063e7-5c98-4b59-8af2-5473669c7075"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac8e0bb9-90c9-4833-8423-d16fde25b4e4") },
		{ "577a0dee-e956-4d99-8fbd-97acdd8ac3df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ProgramBody), "b596af0c-8c70-4c67-a7a4-89f3c7a8cc88"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "5d75f56b-048c-4862-adda-a14a224c08ca"}, { new Terminator(KeywordType.Const), "f55acf61-ed4f-461f-83ee-0e53ce0bd6ba"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "577a0dee-e956-4d99-8fbd-97acdd8ac3df") },
		{ "59d063e7-5c98-4b59-8af2-5473669c7075", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "ecdfa4a0-2f6b-476c-a295-723c2d60ff8d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "59d063e7-5c98-4b59-8af2-5473669c7075") },
		{ "b596af0c-8c70-4c67-a7a4-89f3c7a8cc88", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Period), "ad01e9f1-bad7-4bcf-9dca-805ab2171806"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b596af0c-8c70-4c67-a7a4-89f3c7a8cc88") },
		{ "5d75f56b-048c-4862-adda-a14a224c08ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "c6ce8b6f-a592-4a31-ba67-4aec25a67cbf"}, { new Terminator(KeywordType.Var), "3426388e-1ef0-4441-8b6c-137b3ff20e15"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "5d75f56b-048c-4862-adda-a14a224c08ca") },
		{ "f55acf61-ed4f-461f-83ee-0e53ce0bd6ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "5bcb98b1-6c7b-4044-b5d8-b7b3ea23b87a"}, { Terminator.IdentifierTerminator, "25b20cba-1c07-4cbe-b5d0-b0d8c34bbca2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f55acf61-ed4f-461f-83ee-0e53ce0bd6ba") },
		{ "ecdfa4a0-2f6b-476c-a295-723c2d60ff8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "b3d5ad96-ee5b-46d0-b2f4-514a739f5103"}, { new Terminator(DelimiterType.Comma), "9e52587c-5111-4ddb-94d4-a05b5d00b197"}, { new Terminator(DelimiterType.Colon), "4425a57e-bfcf-40cd-b071-9983c243fd62"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ecdfa4a0-2f6b-476c-a295-723c2d60ff8d") },
		{ "ad01e9f1-bad7-4bcf-9dca-805ab2171806", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.EndTerminator, new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramStruct))}, }, "ad01e9f1-bad7-4bcf-9dca-805ab2171806") },
		{ "c6ce8b6f-a592-4a31-ba67-4aec25a67cbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramDeclarations), "0608de2c-3663-448f-a3f5-dc4f063ea290"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(0, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "c6ce8b6f-a592-4a31-ba67-4aec25a67cbf") },
		{ "3426388e-1ef0-4441-8b6c-137b3ff20e15", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "057e302c-ad0f-4420-b83e-b1ffa782e661"}, { Terminator.IdentifierTerminator, "280b6b74-5a4e-4937-b054-e8f1b11b44a9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3426388e-1ef0-4441-8b6c-137b3ff20e15") },
		{ "5bcb98b1-6c7b-4044-b5d8-b7b3ea23b87a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "0aa3f83d-77eb-4fe5-bc48-ecdab97b11a5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5bcb98b1-6c7b-4044-b5d8-b7b3ea23b87a") },
		{ "25b20cba-1c07-4cbe-b5d0-b0d8c34bbca2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "84f6795f-5b58-4618-af2a-39d70eed2b87"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25b20cba-1c07-4cbe-b5d0-b0d8c34bbca2") },
		{ "b3d5ad96-ee5b-46d0-b2f4-514a739f5103", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f8793bc9-e468-4851-a691-f305c9d78233"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b3d5ad96-ee5b-46d0-b2f4-514a739f5103") },
		{ "9e52587c-5111-4ddb-94d4-a05b5d00b197", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "ac4e077d-f788-4823-b63f-e4e05048f027"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9e52587c-5111-4ddb-94d4-a05b5d00b197") },
		{ "4425a57e-bfcf-40cd-b071-9983c243fd62", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "504222f6-9d09-4476-aef4-cdd6ac44b23b"}, { new NonTerminator(NonTerminatorType.BasicType), "c92add7c-ff3d-4ede-90c4-10219694083b"}, { new Terminator(KeywordType.Array), "4657db25-505c-4877-8063-94cc653d883c"}, { new Terminator(KeywordType.Integer), "569fea77-5df5-4718-a037-ebf833537bf1"}, { new Terminator(KeywordType.Real), "cab00cd6-e738-4376-b34c-43ed1332a950"}, { new Terminator(KeywordType.Boolean), "14f66661-e8a9-45fe-8089-23d0b7358194"}, { new Terminator(KeywordType.Character), "a8d781f8-e7f2-4d98-9b0d-38ae3572acb4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4425a57e-bfcf-40cd-b071-9983c243fd62") },
		{ "0608de2c-3663-448f-a3f5-dc4f063ea290", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "d5184082-64ca-4fa9-b8f1-ee4efddec45b"}, { new Terminator(KeywordType.Begin), "f7f3f0e0-a7c8-475e-9619-7c9c6084da8c"}, { new NonTerminator(NonTerminatorType.Subprogram), "f3a1febf-50b7-4a35-99a2-9fb5b66f400c"}, { new NonTerminator(NonTerminatorType.SubprogramHead), "356af75d-b822-4684-b106-3bcf0440cd11"}, { new Terminator(KeywordType.Procedure), "d6cd14f8-dd5b-4400-931d-3dec9a3fdac5"}, { new Terminator(KeywordType.Function), "2e65a4b7-b6de-4e60-88c4-bf24a725b30f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0608de2c-3663-448f-a3f5-dc4f063ea290") },
		{ "057e302c-ad0f-4420-b83e-b1ffa782e661", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "464aa67f-4ee7-4e66-8187-02e097f4f0a0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "057e302c-ad0f-4420-b83e-b1ffa782e661") },
		{ "280b6b74-5a4e-4937-b054-e8f1b11b44a9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "558c4f76-a686-4e3e-b504-852457072b68"}, { new Terminator(DelimiterType.Comma), "ae79a553-bf53-4d8b-b03c-5877d4f75839"}, { new Terminator(DelimiterType.Colon), "117374a0-c24c-4af2-90a9-311a3ae32c29"},}, new Dictionary<Terminator, ReduceInformation>{ }, "280b6b74-5a4e-4937-b054-e8f1b11b44a9") },
		{ "0aa3f83d-77eb-4fe5-bc48-ecdab97b11a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "09b6a8d2-888d-452e-ad86-0c969667bd9e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "0aa3f83d-77eb-4fe5-bc48-ecdab97b11a5") },
		{ "84f6795f-5b58-4618-af2a-39d70eed2b87", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "3aa43172-e410-4737-b649-f27a1ba97921"}, { new Terminator(OperatorType.Plus), "5888495c-8446-4ec8-96eb-c4cdfb0bd6a0"}, { new Terminator(OperatorType.Minus), "64974682-989b-498d-9e68-9b395805b4e6"}, { Terminator.NumberTerminator, "126f6c5e-da0c-4677-a142-2f39ae846df8"}, { Terminator.CharacterTerminator, "c55e0eac-aef4-4cf6-88c7-9179144bc94c"}, { new Terminator(KeywordType.True), "c772c7a4-8711-49c0-8087-5075cdfe0f5d"}, { new Terminator(KeywordType.False), "40101384-1db0-414d-81af-7ad77e249a70"},}, new Dictionary<Terminator, ReduceInformation>{ }, "84f6795f-5b58-4618-af2a-39d70eed2b87") },
		{ "f8793bc9-e468-4851-a691-f305c9d78233", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ProgramHead))}, }, "f8793bc9-e468-4851-a691-f305c9d78233") },
		{ "ac4e077d-f788-4823-b63f-e4e05048f027", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "75b75d4e-46f6-4ac3-ad4a-043c9852f8b9"}, { new Terminator(DelimiterType.Comma), "9e52587c-5111-4ddb-94d4-a05b5d00b197"}, { new Terminator(DelimiterType.Colon), "4425a57e-bfcf-40cd-b071-9983c243fd62"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ac4e077d-f788-4823-b63f-e4e05048f027") },
		{ "504222f6-9d09-4476-aef4-cdd6ac44b23b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "504222f6-9d09-4476-aef4-cdd6ac44b23b") },
		{ "c92add7c-ff3d-4ede-90c4-10219694083b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "c92add7c-ff3d-4ede-90c4-10219694083b") },
		{ "4657db25-505c-4877-8063-94cc653d883c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "d14de8ba-c5a3-4571-a24c-9461015ccef5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4657db25-505c-4877-8063-94cc653d883c") },
		{ "569fea77-5df5-4718-a037-ebf833537bf1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "569fea77-5df5-4718-a037-ebf833537bf1") },
		{ "cab00cd6-e738-4376-b34c-43ed1332a950", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "cab00cd6-e738-4376-b34c-43ed1332a950") },
		{ "14f66661-e8a9-45fe-8089-23d0b7358194", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "14f66661-e8a9-45fe-8089-23d0b7358194") },
		{ "a8d781f8-e7f2-4d98-9b0d-38ae3572acb4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "a8d781f8-e7f2-4d98-9b0d-38ae3572acb4") },
		{ "d5184082-64ca-4fa9-b8f1-ee4efddec45b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProgramBody))}, }, "d5184082-64ca-4fa9-b8f1-ee4efddec45b") },
		{ "f7f3f0e0-a7c8-475e-9619-7c9c6084da8c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "dc0f8a89-9f48-4f80-95c9-7f84e5cde36d"}, { new NonTerminator(NonTerminatorType.Statement), "f77dde55-48b4-4353-a3c3-37677c20bd47"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "f7f3f0e0-a7c8-475e-9619-7c9c6084da8c") },
		{ "f3a1febf-50b7-4a35-99a2-9fb5b66f400c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "39c9a22c-d37b-4c5c-a4a8-f6d6b537943a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f3a1febf-50b7-4a35-99a2-9fb5b66f400c") },
		{ "356af75d-b822-4684-b106-3bcf0440cd11", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "aba17088-2922-42e6-900e-67e50c36c442"},}, new Dictionary<Terminator, ReduceInformation>{ }, "356af75d-b822-4684-b106-3bcf0440cd11") },
		{ "d6cd14f8-dd5b-4400-931d-3dec9a3fdac5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "d932d3de-4f64-4208-a5bc-945e23000ee3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6cd14f8-dd5b-4400-931d-3dec9a3fdac5") },
		{ "2e65a4b7-b6de-4e60-88c4-bf24a725b30f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "3487a814-f0e4-411c-812e-8fa822639b75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2e65a4b7-b6de-4e60-88c4-bf24a725b30f") },
		{ "464aa67f-4ee7-4e66-8187-02e097f4f0a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "3bf41f0d-a029-47ff-be8d-6c9bed085d4c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "464aa67f-4ee7-4e66-8187-02e097f4f0a0") },
		{ "558c4f76-a686-4e3e-b504-852457072b68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "558c4f76-a686-4e3e-b504-852457072b68") },
		{ "ae79a553-bf53-4d8b-b03c-5877d4f75839", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "67bd0348-de26-4a7a-830f-57dcaee6d804"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae79a553-bf53-4d8b-b03c-5877d4f75839") },
		{ "117374a0-c24c-4af2-90a9-311a3ae32c29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "d272d3e0-2b60-4677-8ce5-1abc2a862329"}, { new NonTerminator(NonTerminatorType.BasicType), "b9609cb2-72af-4e2d-89bb-f3df73b9a643"}, { new Terminator(KeywordType.Array), "f11a29f5-3585-4afd-aae1-c3d3ea221f9d"}, { new Terminator(KeywordType.Integer), "53fb2178-3ce3-41bf-b111-74899ad57bab"}, { new Terminator(KeywordType.Real), "f0bab41c-04a7-4c56-b111-ef249f8f888c"}, { new Terminator(KeywordType.Boolean), "ce45f10b-d787-4ea7-9a1b-be114506cc7b"}, { new Terminator(KeywordType.Character), "687ec48b-b819-4790-a279-957de0b38494"},}, new Dictionary<Terminator, ReduceInformation>{ }, "117374a0-c24c-4af2-90a9-311a3ae32c29") },
		{ "09b6a8d2-888d-452e-ad86-0c969667bd9e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Equal), "26c45f31-d114-4001-bef8-d32585b6b7ce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "09b6a8d2-888d-452e-ad86-0c969667bd9e") },
		{ "3aa43172-e410-4737-b649-f27a1ba97921", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "3aa43172-e410-4737-b649-f27a1ba97921") },
		{ "5888495c-8446-4ec8-96eb-c4cdfb0bd6a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "a237837c-8331-4711-bc1c-8203a0581f75"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5888495c-8446-4ec8-96eb-c4cdfb0bd6a0") },
		{ "64974682-989b-498d-9e68-9b395805b4e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "499b1bcf-b2be-47b3-b18a-7f50ca48e702"},}, new Dictionary<Terminator, ReduceInformation>{ }, "64974682-989b-498d-9e68-9b395805b4e6") },
		{ "126f6c5e-da0c-4677-a142-2f39ae846df8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "126f6c5e-da0c-4677-a142-2f39ae846df8") },
		{ "c55e0eac-aef4-4cf6-88c7-9179144bc94c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "c55e0eac-aef4-4cf6-88c7-9179144bc94c") },
		{ "c772c7a4-8711-49c0-8087-5075cdfe0f5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "c772c7a4-8711-49c0-8087-5075cdfe0f5d") },
		{ "40101384-1db0-414d-81af-7ad77e249a70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ConstValue))}, }, "40101384-1db0-414d-81af-7ad77e249a70") },
		{ "75b75d4e-46f6-4ac3-ad4a-043c9852f8b9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "75b75d4e-46f6-4ac3-ad4a-043c9852f8b9") },
		{ "d14de8ba-c5a3-4571-a24c-9461015ccef5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "3c043fda-efe1-49cd-ab2a-0b921b618c65"}, { Terminator.NumberTerminator, "1d59d28d-3de6-4b9b-aec2-b237c4a2e6dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d14de8ba-c5a3-4571-a24c-9461015ccef5") },
		{ "dc0f8a89-9f48-4f80-95c9-7f84e5cde36d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "6dc9b39a-ee80-4f6e-8e63-5af2f232ce73"}, { new Terminator(DelimiterType.Semicolon), "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dc0f8a89-9f48-4f80-95c9-7f84e5cde36d") },
		{ "f77dde55-48b4-4353-a3c3-37677c20bd47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.StatementList))}, }, "f77dde55-48b4-4353-a3c3-37677c20bd47") },
		{ "ae30128e-efd3-4103-9828-59d8da47b869", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "754b1e8a-2dd0-427b-8432-8803da25365a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae30128e-efd3-4103-9828-59d8da47b869") },
		{ "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5") },
		{ "cc93b19c-954a-4e6d-9ffe-b1d796b56b27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "cc93b19c-954a-4e6d-9ffe-b1d796b56b27") },
		{ "bd6a5146-d6d4-453b-9442-d2d146452e94", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9fc8abd8-8d69-4d11-a3b6-4dda6ed5b94b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "4dde76cc-98f5-4ad2-b620-4bf0784bdfc2"}, { new NonTerminator(NonTerminatorType.Term), "2231c4a1-166c-4a9d-a192-0a6f61c69e0a"}, { new NonTerminator(NonTerminatorType.Factor), "15ec65fb-8226-4e0b-9eb4-cdcb95aa1bbf"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bd6a5146-d6d4-453b-9442-d2d146452e94") },
		{ "138ba7eb-e0e3-403c-a842-2268411e11cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "568c369e-15d8-487c-9b8d-701dfd8e28f3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "138ba7eb-e0e3-403c-a842-2268411e11cc") },
		{ "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "e144fbb1-95ba-42de-9e67-0cbe04e38065"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "605e6610-c7fd-4688-914d-554d4e1f0467"}, { new NonTerminator(NonTerminatorType.Term), "ecf78c42-792d-4d48-8f88-e1cd8e101dd0"}, { new NonTerminator(NonTerminatorType.Factor), "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad") },
		{ "b128ff00-00cc-46f4-b962-7e7174f1fb28", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdVarPart), "cb2faf98-22c9-41de-9363-062d1cd8b143"}, { new Terminator(DelimiterType.LeftSquareBracket), "1d4b60ea-bd48-4214-88b6-e64152da03fa"}, { new Terminator(DelimiterType.LeftParenthesis), "99786a06-52c0-4235-a04f-65a037bae8ec"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "b128ff00-00cc-46f4-b962-7e7174f1fb28") },
		{ "ac1bf4c3-ab39-4ece-a684-e40886b7c49d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "d4b9257d-cc97-4168-9b85-787d8172ccaa"}, { new NonTerminator(NonTerminatorType.Statement), "f77dde55-48b4-4353-a3c3-37677c20bd47"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "ac1bf4c3-ab39-4ece-a684-e40886b7c49d") },
		{ "39c9a22c-d37b-4c5c-a4a8-f6d6b537943a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Procedure), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, { new Terminator(KeywordType.Function), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramDeclarations))}, }, "39c9a22c-d37b-4c5c-a4a8-f6d6b537943a") },
		{ "aba17088-2922-42e6-900e-67e50c36c442", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SubprogramBody), "46a2a510-ef6d-49d8-aac8-b3249b8a0f30"}, { new NonTerminator(NonTerminatorType.ConstDeclarations), "7fab4367-5d13-48d0-9402-2701a9f12ae1"}, { new Terminator(KeywordType.Const), "ff0a937b-3380-47bd-90c1-6932016e5c1a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "aba17088-2922-42e6-900e-67e50c36c442") },
		{ "d932d3de-4f64-4208-a5bc-945e23000ee3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "080fca0d-0105-4281-9d99-4876e0d13c4a"}, { new Terminator(DelimiterType.LeftParenthesis), "8b65673e-a3a5-4123-8ff9-75cea90a1e23"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "d932d3de-4f64-4208-a5bc-945e23000ee3") },
		{ "3487a814-f0e4-411c-812e-8fa822639b75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.FormalParameter), "782b3e4a-b18e-42cc-a837-e39cb05c923c"}, { new Terminator(DelimiterType.LeftParenthesis), "94dea7a4-91c7-4aee-810c-d714e7c94aac"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "3487a814-f0e4-411c-812e-8fa822639b75") },
		{ "3bf41f0d-a029-47ff-be8d-6c9bed085d4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "bf18e59e-ede6-4df1-b6cc-b8f01e241d5a"}, { new Terminator(DelimiterType.Comma), "ae79a553-bf53-4d8b-b03c-5877d4f75839"}, { new Terminator(DelimiterType.Colon), "117374a0-c24c-4af2-90a9-311a3ae32c29"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3bf41f0d-a029-47ff-be8d-6c9bed085d4c") },
		{ "67bd0348-de26-4a7a-830f-57dcaee6d804", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "a2bb433e-490b-421c-900c-9ee2bb1ba573"}, { new Terminator(DelimiterType.Comma), "ae79a553-bf53-4d8b-b03c-5877d4f75839"}, { new Terminator(DelimiterType.Colon), "117374a0-c24c-4af2-90a9-311a3ae32c29"},}, new Dictionary<Terminator, ReduceInformation>{ }, "67bd0348-de26-4a7a-830f-57dcaee6d804") },
		{ "d272d3e0-2b60-4677-8ce5-1abc2a862329", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "d272d3e0-2b60-4677-8ce5-1abc2a862329") },
		{ "b9609cb2-72af-4e2d-89bb-f3df73b9a643", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "b9609cb2-72af-4e2d-89bb-f3df73b9a643") },
		{ "f11a29f5-3585-4afd-aae1-c3d3ea221f9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "8ce41908-5fa9-4090-9f73-0619c5cdd916"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f11a29f5-3585-4afd-aae1-c3d3ea221f9d") },
		{ "53fb2178-3ce3-41bf-b111-74899ad57bab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "53fb2178-3ce3-41bf-b111-74899ad57bab") },
		{ "f0bab41c-04a7-4c56-b111-ef249f8f888c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "f0bab41c-04a7-4c56-b111-ef249f8f888c") },
		{ "ce45f10b-d787-4ea7-9a1b-be114506cc7b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "ce45f10b-d787-4ea7-9a1b-be114506cc7b") },
		{ "687ec48b-b819-4790-a279-957de0b38494", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "687ec48b-b819-4790-a279-957de0b38494") },
		{ "26c45f31-d114-4001-bef8-d32585b6b7ce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstValue), "e3658ebd-2cd0-44b8-998c-026dd9283b29"}, { new Terminator(OperatorType.Plus), "5888495c-8446-4ec8-96eb-c4cdfb0bd6a0"}, { new Terminator(OperatorType.Minus), "64974682-989b-498d-9e68-9b395805b4e6"}, { Terminator.NumberTerminator, "126f6c5e-da0c-4677-a142-2f39ae846df8"}, { Terminator.CharacterTerminator, "c55e0eac-aef4-4cf6-88c7-9179144bc94c"}, { new Terminator(KeywordType.True), "c772c7a4-8711-49c0-8087-5075cdfe0f5d"}, { new Terminator(KeywordType.False), "40101384-1db0-414d-81af-7ad77e249a70"},}, new Dictionary<Terminator, ReduceInformation>{ }, "26c45f31-d114-4001-bef8-d32585b6b7ce") },
		{ "a237837c-8331-4711-bc1c-8203a0581f75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "a237837c-8331-4711-bc1c-8203a0581f75") },
		{ "499b1bcf-b2be-47b3-b18a-7f50ca48e702", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ConstValue))}, }, "499b1bcf-b2be-47b3-b18a-7f50ca48e702") },
		{ "3c043fda-efe1-49cd-ab2a-0b921b618c65", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "e2623b85-81c9-44d6-9563-4c8a0b65e302"}, { new Terminator(DelimiterType.Comma), "c2bd4f6a-1204-4250-9472-219492cfb89b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3c043fda-efe1-49cd-ab2a-0b921b618c65") },
		{ "1d59d28d-3de6-4b9b-aec2-b237c4a2e6dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "370fcba4-83a3-4921-b115-bb26e6871285"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d59d28d-3de6-4b9b-aec2-b237c4a2e6dc") },
		{ "6dc9b39a-ee80-4f6e-8e63-5af2f232ce73", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Period), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "6dc9b39a-ee80-4f6e-8e63-5af2f232ce73") },
		{ "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "684a67d4-c18e-48e7-afc1-9a0a0d1e1752"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4") },
		{ "754b1e8a-2dd0-427b-8432-8803da25365a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8f382f5b-6c98-4b81-b87f-de546d611e90"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "e6cbc19e-7b86-4c78-9ece-4d6bcbfac85a"}, { new NonTerminator(NonTerminatorType.Term), "1122c8c0-4d87-47c1-9d0e-00be92c85d2b"}, { new NonTerminator(NonTerminatorType.Factor), "0ac68032-8218-4afe-a892-68fe4b01ea75"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "754b1e8a-2dd0-427b-8432-8803da25365a") },
		{ "9fc8abd8-8d69-4d11-a3b6-4dda6ed5b94b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "5fc21166-21d1-429b-8e1a-99eaff04d94e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9fc8abd8-8d69-4d11-a3b6-4dda6ed5b94b") },
		{ "4dde76cc-98f5-4ad2-b620-4bf0784bdfc2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "6e573b43-71e9-476b-b719-f38a9f200ede"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "455280c0-2150-41fd-be99-25f1605f5f2c"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "4dde76cc-98f5-4ad2-b620-4bf0784bdfc2") },
		{ "2231c4a1-166c-4a9d-a192-0a6f61c69e0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fe7976c2-8b96-4a3c-8d9c-d39df033d42f"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2231c4a1-166c-4a9d-a192-0a6f61c69e0a") },
		{ "15ec65fb-8226-4e0b-9eb4-cdcb95aa1bbf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "15ec65fb-8226-4e0b-9eb4-cdcb95aa1bbf") },
		{ "6ee62975-5e32-49ab-a007-124b7506c1dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "6ee62975-5e32-49ab-a007-124b7506c1dc") },
		{ "bac3ff40-f069-4e49-b93e-49f067a6fffe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bac3ff40-f069-4e49-b93e-49f067a6fffe") },
		{ "d33fe735-f418-4cd6-8430-d08c745447a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "9ea9d11f-c1a3-4fc3-be39-28074fa32344"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d33fe735-f418-4cd6-8430-d08c745447a6") },
		{ "d7196c33-a63c-4a03-b4e4-c03bb0237729", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "5b31c571-6115-461a-b87a-d13ed77489a6"}, { new NonTerminator(NonTerminatorType.IdVarPart), "39fb0cb2-d4a7-496b-a31d-5819688d93e1"}, { new Terminator(DelimiterType.LeftSquareBracket), "0135a0ef-9913-4d0c-af3c-542c2431e2a0"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d7196c33-a63c-4a03-b4e4-c03bb0237729") },
		{ "41932ced-262a-4a2d-8366-fba91da5a371", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "dfb5c168-e5b9-4805-b28f-e420d0629c18"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "41932ced-262a-4a2d-8366-fba91da5a371") },
		{ "73ba7b97-7970-4d14-864f-82e10cee8622", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1eb8a073-fb7d-4ee8-abfb-17c4c0640f6c"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73ba7b97-7970-4d14-864f-82e10cee8622") },
		{ "340cb8fb-7fab-4d27-9190-b9d470c24232", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c2070796-4057-4fdf-b63f-7b2a3fdf12b6"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "340cb8fb-7fab-4d27-9190-b9d470c24232") },
		{ "a9a49511-4cbd-4f2d-9f8e-d7c14498737d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a9a49511-4cbd-4f2d-9f8e-d7c14498737d") },
		{ "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d") },
		{ "568c369e-15d8-487c-9b8d-701dfd8e28f3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "76b175a2-a316-4897-b378-efd7d8346267"},}, new Dictionary<Terminator, ReduceInformation>{ }, "568c369e-15d8-487c-9b8d-701dfd8e28f3") },
		{ "e144fbb1-95ba-42de-9e67-0cbe04e38065", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "c3a7155c-462e-4f12-a238-df352baf278b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e144fbb1-95ba-42de-9e67-0cbe04e38065") },
		{ "605e6610-c7fd-4688-914d-554d4e1f0467", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "262723e9-f656-4fa7-a18f-41562428e4c1"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "7aa132fa-db77-4325-905c-f44864489918"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "605e6610-c7fd-4688-914d-554d4e1f0467") },
		{ "ecf78c42-792d-4d48-8f88-e1cd8e101dd0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1c13de51-4b5f-47e8-86e1-0179943b6887"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ecf78c42-792d-4d48-8f88-e1cd8e101dd0") },
		{ "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1") },
		{ "2b65415f-fb75-427d-a310-39b44b8db03a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2b65415f-fb75-427d-a310-39b44b8db03a") },
		{ "80bf23b2-58db-4d1a-84d2-bec3e73c0429", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "80bf23b2-58db-4d1a-84d2-bec3e73c0429") },
		{ "1cd421b5-16d2-469d-bb62-1a97353ec366", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "ef761f6e-5731-4cc7-a0a5-db341f087d56"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1cd421b5-16d2-469d-bb62-1a97353ec366") },
		{ "db36538c-6d08-4417-a589-ca40963ee1c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "649ddb60-ee22-48b8-8ef2-b4968fd04ec2"}, { new NonTerminator(NonTerminatorType.IdVarPart), "40cadde7-9695-4841-938d-a0be7f9de39c"}, { new Terminator(DelimiterType.LeftSquareBracket), "4526905d-bcf0-4fbc-8ca3-52cde02dfb30"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "db36538c-6d08-4417-a589-ca40963ee1c5") },
		{ "47a918c3-1ad6-4fb3-aa15-e10f85953c5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "5ac6d830-5ebb-41ec-87f2-626df0b42f6a"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "47a918c3-1ad6-4fb3-aa15-e10f85953c5d") },
		{ "4a9f4005-1b1b-445e-905a-389a24c88d05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "58a91787-22ce-4338-990f-f95dd112537d"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4a9f4005-1b1b-445e-905a-389a24c88d05") },
		{ "d6df8a91-452b-46a5-b35e-18cb2be98261", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "af0aa7a6-9012-4843-838d-d824103f506e"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6df8a91-452b-46a5-b35e-18cb2be98261") },
		{ "c9085b14-74b4-4630-8f90-1009eb9f7674", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c9085b14-74b4-4630-8f90-1009eb9f7674") },
		{ "44c59546-5b4e-48b3-a3eb-de499bb1a988", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "44c59546-5b4e-48b3-a3eb-de499bb1a988") },
		{ "cb2faf98-22c9-41de-9363-062d1cd8b143", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "cb2faf98-22c9-41de-9363-062d1cd8b143") },
		{ "1d4b60ea-bd48-4214-88b6-e64152da03fa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "766f234b-2b99-4e74-8468-c9c095d55b70"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1d4b60ea-bd48-4214-88b6-e64152da03fa") },
		{ "99786a06-52c0-4235-a04f-65a037bae8ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "cb29a8e8-f913-4340-b41e-f64c539267b0"}, { new NonTerminator(NonTerminatorType.ExpressionList), "85e7e371-cbb3-4d90-aeb1-0e3e3c879ce0"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "99786a06-52c0-4235-a04f-65a037bae8ec") },
		{ "d4b9257d-cc97-4168-9b85-787d8172ccaa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "ef0634e1-0d3f-41f0-bff3-57bf4e2884e6"}, { new Terminator(DelimiterType.Semicolon), "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d4b9257d-cc97-4168-9b85-787d8172ccaa") },
		{ "46a2a510-ef6d-49d8-aac8-b3249b8a0f30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Subprogram))}, }, "46a2a510-ef6d-49d8-aac8-b3249b8a0f30") },
		{ "7fab4367-5d13-48d0-9402-2701a9f12ae1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclarations), "faad7a30-d814-4fd5-90e8-c086f9a0e648"}, { new Terminator(KeywordType.Var), "03ea86f5-4260-4120-901a-b29f96c9e9f2"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(0, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "7fab4367-5d13-48d0-9402-2701a9f12ae1") },
		{ "ff0a937b-3380-47bd-90c1-6932016e5c1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ConstDeclaration), "93550301-a8ab-4493-8423-46cc83abe0cd"}, { Terminator.IdentifierTerminator, "25b20cba-1c07-4cbe-b5d0-b0d8c34bbca2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ff0a937b-3380-47bd-90c1-6932016e5c1a") },
		{ "080fca0d-0105-4281-9d99-4876e0d13c4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "080fca0d-0105-4281-9d99-4876e0d13c4a") },
		{ "8b65673e-a3a5-4123-8ff9-75cea90a1e23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f038691f-4133-4728-bd63-eedbb65b0a7e"}, { new NonTerminator(NonTerminatorType.ParameterList), "f29e8573-114f-49a5-9d6b-6315292319b4"}, { new NonTerminator(NonTerminatorType.Parameter), "ab3d74e2-19c7-4f7e-8d3c-9826e996d18f"}, { new NonTerminator(NonTerminatorType.VarParameter), "a1cc7797-e56e-43e0-b3dd-dcb9ce0116cc"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7007b3eb-8197-4235-a415-cb0afeb5e530"}, { new Terminator(KeywordType.Var), "a6f74335-800b-4ee9-bd3c-0ce9bcfc3612"}, { Terminator.IdentifierTerminator, "3725ab6a-0044-4165-85f8-852735c5eed3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8b65673e-a3a5-4123-8ff9-75cea90a1e23") },
		{ "782b3e4a-b18e-42cc-a837-e39cb05c923c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Colon), "59527a1a-de12-484e-9461-0b5863383f5b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "782b3e4a-b18e-42cc-a837-e39cb05c923c") },
		{ "94dea7a4-91c7-4aee-810c-d714e7c94aac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c1dde178-9eec-47d8-8180-fc87cc1865dc"}, { new NonTerminator(NonTerminatorType.ParameterList), "77142180-a1da-40cd-b722-e9d2d01d1fa4"}, { new NonTerminator(NonTerminatorType.Parameter), "ab3d74e2-19c7-4f7e-8d3c-9826e996d18f"}, { new NonTerminator(NonTerminatorType.VarParameter), "a1cc7797-e56e-43e0-b3dd-dcb9ce0116cc"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7007b3eb-8197-4235-a415-cb0afeb5e530"}, { new Terminator(KeywordType.Var), "a6f74335-800b-4ee9-bd3c-0ce9bcfc3612"}, { Terminator.IdentifierTerminator, "3725ab6a-0044-4165-85f8-852735c5eed3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "94dea7a4-91c7-4aee-810c-d714e7c94aac") },
		{ "bf18e59e-ede6-4df1-b6cc-b8f01e241d5a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.VarDeclaration))}, }, "bf18e59e-ede6-4df1-b6cc-b8f01e241d5a") },
		{ "a2bb433e-490b-421c-900c-9ee2bb1ba573", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "a2bb433e-490b-421c-900c-9ee2bb1ba573") },
		{ "8ce41908-5fa9-4090-9f73-0619c5cdd916", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "cf4d7c50-28fd-4021-8a19-ad8fa3f37c67"}, { Terminator.NumberTerminator, "1d59d28d-3de6-4b9b-aec2-b237c4a2e6dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8ce41908-5fa9-4090-9f73-0619c5cdd916") },
		{ "e3658ebd-2cd0-44b8-998c-026dd9283b29", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.ConstDeclaration))}, }, "e3658ebd-2cd0-44b8-998c-026dd9283b29") },
		{ "e2623b85-81c9-44d6-9563-4c8a0b65e302", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "1332454e-b3a5-4813-b6e5-e5f72f7304b5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e2623b85-81c9-44d6-9563-4c8a0b65e302") },
		{ "c2bd4f6a-1204-4250-9472-219492cfb89b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "cba2bd3d-a0e0-4d1d-9cf3-d7e7cc4c608d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c2bd4f6a-1204-4250-9472-219492cfb89b") },
		{ "370fcba4-83a3-4921-b115-bb26e6871285", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "53aabf52-fdfd-4fa4-9450-bbf3ef645f23"},}, new Dictionary<Terminator, ReduceInformation>{ }, "370fcba4-83a3-4921-b115-bb26e6871285") },
		{ "684a67d4-c18e-48e7-afc1-9a0a0d1e1752", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.StatementList))}, }, "684a67d4-c18e-48e7-afc1-9a0a0d1e1752") },
		{ "8f382f5b-6c98-4b81-b87f-de546d611e90", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "8f382f5b-6c98-4b81-b87f-de546d611e90") },
		{ "e6cbc19e-7b86-4c78-9ece-4d6bcbfac85a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "2fd865f8-145a-4a09-bc6a-f8e392402031"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "e31aba3d-0343-4f1d-a8f0-9230001b9231"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "e6cbc19e-7b86-4c78-9ece-4d6bcbfac85a") },
		{ "1122c8c0-4d87-47c1-9d0e-00be92c85d2b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "94df7fe4-5160-4cd6-b591-b6aed9f9680b"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "1122c8c0-4d87-47c1-9d0e-00be92c85d2b") },
		{ "0ac68032-8218-4afe-a892-68fe4b01ea75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "0ac68032-8218-4afe-a892-68fe4b01ea75") },
		{ "8372929f-18a1-42eb-9c56-cdf6b496ab7f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "8372929f-18a1-42eb-9c56-cdf6b496ab7f") },
		{ "5eb809a9-364a-4c59-b743-12bff178dcb4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "5eb809a9-364a-4c59-b743-12bff178dcb4") },
		{ "31c16865-7977-428f-b0d4-ae5d3ddaf083", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "01e94f98-621a-4e26-b0a8-6bac22d690e0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "31c16865-7977-428f-b0d4-ae5d3ddaf083") },
		{ "7c517b7f-b3b9-449d-b2e2-4029db449b2b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "6f1aca56-c74b-4803-98a4-2a38b4b14c09"}, { new NonTerminator(NonTerminatorType.IdVarPart), "37f959d9-f414-48e7-8128-443906eb3249"}, { new Terminator(DelimiterType.LeftSquareBracket), "52b6e6a9-452f-4f0e-96b1-6dc918e7e161"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "7c517b7f-b3b9-449d-b2e2-4029db449b2b") },
		{ "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7222e8f0-4384-41a2-833b-2748ce5171c4"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b") },
		{ "40916387-5e18-4fcb-83c5-5f1c3d466e52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d19d36d4-dbf0-46a2-9284-a3714ae698c0"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "40916387-5e18-4fcb-83c5-5f1c3d466e52") },
		{ "7f3f76ea-4668-486a-a787-4241ae9fff0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "30e8b3a8-57ed-4c58-a634-a666a35838f8"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7f3f76ea-4668-486a-a787-4241ae9fff0f") },
		{ "d99f0be4-ea5d-4e5e-b704-9b538b366113", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d99f0be4-ea5d-4e5e-b704-9b538b366113") },
		{ "a319032c-8e25-4276-a480-396f2aeae3a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a319032c-8e25-4276-a480-396f2aeae3a6") },
		{ "5fc21166-21d1-429b-8e1a-99eaff04d94e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "84cb6a9e-2732-4e1b-8b6d-9c06fb3ca7b4"}, { new NonTerminator(NonTerminatorType.Variable), "2003d515-90bb-4516-b5f4-2a4cedf0fad3"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "39958051-a99c-49c9-8d22-79d2456b0c78"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "f3c84ef3-69cf-4881-822f-d2a6f0a8f160"}, { new Terminator(KeywordType.If), "2436839b-20b6-4147-af90-9c877a35feea"}, { new Terminator(KeywordType.For), "6afb4fc5-5847-4ac1-8489-830a122891f2"}, { new Terminator(KeywordType.While), "3e60e08c-33fc-4674-961c-18019584f9ba"}, { Terminator.IdentifierTerminator, "24a23c8f-9000-443b-9fd0-cb6464a74efa"}, { new Terminator(KeywordType.Begin), "554d0078-799c-4218-9588-712cf3fe3c14"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "5fc21166-21d1-429b-8e1a-99eaff04d94e") },
		{ "6e573b43-71e9-476b-b719-f38a9f200ede", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "6b1864a6-0e6d-4ead-8f5d-5d65a05384cb"}, { new NonTerminator(NonTerminatorType.Term), "2b50054b-9d22-4a3f-8301-d4ac1b214b4c"}, { new NonTerminator(NonTerminatorType.Factor), "d8983d6e-a324-42b1-abda-e5e9b48f6c68"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6e573b43-71e9-476b-b719-f38a9f200ede") },
		{ "59e0b42f-0f19-4905-bda8-027e4043f4af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "59e0b42f-0f19-4905-bda8-027e4043f4af") },
		{ "7863edae-4a97-4391-96b2-2e884ec9446a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "7863edae-4a97-4391-96b2-2e884ec9446a") },
		{ "16951dba-a455-4455-ae09-e1f402e42c72", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "16951dba-a455-4455-ae09-e1f402e42c72") },
		{ "e510cf8d-4628-45cd-aea4-8b27b1e67c4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "e510cf8d-4628-45cd-aea4-8b27b1e67c4a") },
		{ "59406bfc-b725-4683-80e7-445c998f005a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "59406bfc-b725-4683-80e7-445c998f005a") },
		{ "8f8ba332-6060-40ce-9e86-bdb7a7e3f793", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.RelationOperator))}, }, "8f8ba332-6060-40ce-9e86-bdb7a7e3f793") },
		{ "455280c0-2150-41fd-be99-25f1605f5f2c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "37371454-69a0-4bb3-b072-1bc567364aec"}, { new NonTerminator(NonTerminatorType.Factor), "15ec65fb-8226-4e0b-9eb4-cdcb95aa1bbf"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "455280c0-2150-41fd-be99-25f1605f5f2c") },
		{ "ff07e7b0-2318-47ab-b5b3-62d38c00a279", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "ff07e7b0-2318-47ab-b5b3-62d38c00a279") },
		{ "5df57e00-07dc-4d54-9421-a3d931a3df6f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "5df57e00-07dc-4d54-9421-a3d931a3df6f") },
		{ "81a1e4b9-aff3-4e40-8c7b-890e05350f4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.AddOperator))}, }, "81a1e4b9-aff3-4e40-8c7b-890e05350f4a") },
		{ "fe7976c2-8b96-4a3c-8d9c-d39df033d42f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2323e297-0ba6-44ce-a116-44cf128b60da"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fe7976c2-8b96-4a3c-8d9c-d39df033d42f") },
		{ "f804a71b-b820-43aa-be44-f8a295bde558", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "f804a71b-b820-43aa-be44-f8a295bde558") },
		{ "8b737830-70f6-4bec-97a5-6296481bf768", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "8b737830-70f6-4bec-97a5-6296481bf768") },
		{ "3505d411-753d-4ad5-baa6-5ce42611bdf5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "3505d411-753d-4ad5-baa6-5ce42611bdf5") },
		{ "12058fc7-adfa-4bfa-8590-10cae139c4ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "12058fc7-adfa-4bfa-8590-10cae139c4ef") },
		{ "43b7afac-75b0-4e50-94f1-52e653362a16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { Terminator.NumberTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { Terminator.IdentifierTerminator, new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(DelimiterType.LeftParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.Not), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.True), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, { new Terminator(KeywordType.False), new ReduceInformation(1, new NonTerminator(NonTerminatorType.MultiplyOperator))}, }, "43b7afac-75b0-4e50-94f1-52e653362a16") },
		{ "9ea9d11f-c1a3-4fc3-be39-28074fa32344", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "168bae91-f78b-49ea-8907-9d29c962343f"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9ea9d11f-c1a3-4fc3-be39-28074fa32344") },
		{ "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "22f51367-b481-4782-9757-0a8e8a79af93"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "8f2909ee-3fc4-4ae4-9d5d-e81b9d5b0989"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c") },
		{ "14e8d50f-f839-47bc-8625-00061a186a05", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2601e0f6-f43b-4d6c-8e4d-e91a592d67b5"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "14e8d50f-f839-47bc-8625-00061a186a05") },
		{ "a556177c-acc4-4abb-843b-1217bc959c0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "a556177c-acc4-4abb-843b-1217bc959c0a") },
		{ "3ea3c950-0614-4e77-875b-198db9b5dec1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "3ea3c950-0614-4e77-875b-198db9b5dec1") },
		{ "25279cd9-10d3-491c-b22b-ab6d219987b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "25279cd9-10d3-491c-b22b-ab6d219987b1") },
		{ "34905681-5c09-48bb-98d9-c7d0f1220b9f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d0e6277a-b96b-4f3f-be22-2991c809852b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "34905681-5c09-48bb-98d9-c7d0f1220b9f") },
		{ "1afab765-abab-4ee5-8e81-afb4057ed142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "86898790-5c7b-4d4b-895f-adfdc2f5926a"}, { new NonTerminator(NonTerminatorType.IdVarPart), "d9fb70ba-3f04-4fd3-a7be-f2d7f64b40fe"}, { new Terminator(DelimiterType.LeftSquareBracket), "a5abf3c5-3044-4350-ba90-8d4b3fce6f52"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1afab765-abab-4ee5-8e81-afb4057ed142") },
		{ "9c2adb6e-9a6a-4732-a2ec-db4f9110891b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3d0615ec-68b6-4d5f-98f0-36c7c5b15ac4"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9c2adb6e-9a6a-4732-a2ec-db4f9110891b") },
		{ "c9c87e1a-2f2c-453e-978b-d36fb68ca681", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7399ceb2-ea9f-4ea2-812d-b3d42b6466d0"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9c87e1a-2f2c-453e-978b-d36fb68ca681") },
		{ "c9b6ac1b-3387-4e86-a381-f2142b65e4b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f7b22a84-64db-4655-8c2d-e51593725b63"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c9b6ac1b-3387-4e86-a381-f2142b65e4b4") },
		{ "407254a3-f259-4d76-959f-cc762a30e5e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "407254a3-f259-4d76-959f-cc762a30e5e7") },
		{ "fbee4454-02b0-44fc-a152-e08a97231e3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fbee4454-02b0-44fc-a152-e08a97231e3a") },
		{ "5b31c571-6115-461a-b87a-d13ed77489a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "867872bf-45ee-4d6b-9769-fd2b57eceba1"}, { new NonTerminator(NonTerminatorType.ExpressionList), "857ad6f5-4e80-4519-a022-19066c1d24cf"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5b31c571-6115-461a-b87a-d13ed77489a6") },
		{ "39fb0cb2-d4a7-496b-a31d-5819688d93e1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "39fb0cb2-d4a7-496b-a31d-5819688d93e1") },
		{ "0135a0ef-9913-4d0c-af3c-542c2431e2a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d6a9be9f-fda9-4c55-a106-80540580a3d5"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0135a0ef-9913-4d0c-af3c-542c2431e2a0") },
		{ "dfb5c168-e5b9-4805-b28f-e420d0629c18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "dfb5c168-e5b9-4805-b28f-e420d0629c18") },
		{ "1eb8a073-fb7d-4ee8-abfb-17c4c0640f6c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1eb8a073-fb7d-4ee8-abfb-17c4c0640f6c") },
		{ "c2070796-4057-4fdf-b63f-7b2a3fdf12b6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c2070796-4057-4fdf-b63f-7b2a3fdf12b6") },
		{ "76b175a2-a316-4897-b378-efd7d8346267", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "a65c3d77-5617-4761-81a9-4c577341a439"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b87c5446-1019-4031-b302-7abb9764f456"}, { new NonTerminator(NonTerminatorType.Term), "9f627a23-8645-4cd3-a1e5-813cf0e39f5e"}, { new NonTerminator(NonTerminatorType.Factor), "277e1067-8bdd-4e98-b081-283debc883c7"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "76b175a2-a316-4897-b378-efd7d8346267") },
		{ "c3a7155c-462e-4f12-a238-df352baf278b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "d1977eb6-08aa-4a64-b5ac-749e2dd30a08"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "c3a7155c-462e-4f12-a238-df352baf278b") },
		{ "262723e9-f656-4fa7-a18f-41562428e4c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "43fb9243-dc99-4eb1-883f-65c6186471ee"}, { new NonTerminator(NonTerminatorType.Term), "59b0e133-71b4-45a3-83c1-8971fe651bcc"}, { new NonTerminator(NonTerminatorType.Factor), "604834f5-b217-434f-9367-7bfed56ab9b0"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "262723e9-f656-4fa7-a18f-41562428e4c1") },
		{ "7aa132fa-db77-4325-905c-f44864489918", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2849ea39-bcb5-4c15-9f21-310c3c1db53e"}, { new NonTerminator(NonTerminatorType.Factor), "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7aa132fa-db77-4325-905c-f44864489918") },
		{ "1c13de51-4b5f-47e8-86e1-0179943b6887", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8dbe4e2c-c11c-45c8-b074-53e60be3ea20"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1c13de51-4b5f-47e8-86e1-0179943b6887") },
		{ "ef761f6e-5731-4cc7-a0a5-db341f087d56", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "080c9bbf-6992-42eb-9283-477c62b29ed8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ef761f6e-5731-4cc7-a0a5-db341f087d56") },
		{ "649ddb60-ee22-48b8-8ef2-b4968fd04ec2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "f6eef27a-a544-438e-9401-ee1d329edfae"}, { new NonTerminator(NonTerminatorType.ExpressionList), "9ec9e564-451f-42b5-8167-b460d9f017e2"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "649ddb60-ee22-48b8-8ef2-b4968fd04ec2") },
		{ "40cadde7-9695-4841-938d-a0be7f9de39c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "40cadde7-9695-4841-938d-a0be7f9de39c") },
		{ "4526905d-bcf0-4fbc-8ca3-52cde02dfb30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "0598be14-ca9b-4e4e-a8f0-fa2b4a328206"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4526905d-bcf0-4fbc-8ca3-52cde02dfb30") },
		{ "5ac6d830-5ebb-41ec-87f2-626df0b42f6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "5ac6d830-5ebb-41ec-87f2-626df0b42f6a") },
		{ "58a91787-22ce-4338-990f-f95dd112537d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "58a91787-22ce-4338-990f-f95dd112537d") },
		{ "af0aa7a6-9012-4843-838d-d824103f506e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "af0aa7a6-9012-4843-838d-d824103f506e") },
		{ "766f234b-2b99-4e74-8468-c9c095d55b70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "39ddeae6-dd11-43a1-9fbc-e64452469d23"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "766f234b-2b99-4e74-8468-c9c095d55b70") },
		{ "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2") },
		{ "2cb1744c-c440-4a4b-96d5-de8bf99406c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "70777a5f-d416-4536-a439-9f6a25b1e3b3"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "02ee4aa2-e583-4265-a92e-a32ddb443116"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "2cb1744c-c440-4a4b-96d5-de8bf99406c5") },
		{ "f995f955-70b5-42d4-8abd-c14d8abf1b85", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "47ffd406-53c9-4960-9673-b91d67c6b194"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f995f955-70b5-42d4-8abd-c14d8abf1b85") },
		{ "e5c504d5-3b0f-40f9-8667-5e2cb304f89b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "e5c504d5-3b0f-40f9-8667-5e2cb304f89b") },
		{ "d0117e0b-f480-459d-8474-d2167983bc1a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d0117e0b-f480-459d-8474-d2167983bc1a") },
		{ "4b621297-900b-4d2b-a080-eb3b76f9622c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4b621297-900b-4d2b-a080-eb3b76f9622c") },
		{ "e696d593-3e5a-469f-ac63-2f4798ddd4c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "59618cd7-7879-4b9f-bc63-65bd9c76777b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e696d593-3e5a-469f-ac63-2f4798ddd4c1") },
		{ "a7aa1b0d-692d-4711-9b1b-bf0beeff9348", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "9032e67e-801b-40b3-b9cb-11e1520e7541"}, { new NonTerminator(NonTerminatorType.IdVarPart), "04b72690-477c-4710-93d3-91c4bba6f269"}, { new Terminator(DelimiterType.LeftSquareBracket), "6c1237a0-4351-4409-93b8-bc3f71130644"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348") },
		{ "9972f94a-15f5-4545-b21e-f6b80df89fc4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c14e7212-7c5d-4721-8bda-06258999dc3f"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9972f94a-15f5-4545-b21e-f6b80df89fc4") },
		{ "25ee3621-0f2e-400f-a56e-b58c6924b370", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "905f6f19-14dd-4f2e-9d43-6a20248a8b88"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "25ee3621-0f2e-400f-a56e-b58c6924b370") },
		{ "c0e51999-5060-441b-b2df-88f948fae74d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "78dc179a-8471-4472-ae09-e6d7aeaf2a4e"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c0e51999-5060-441b-b2df-88f948fae74d") },
		{ "0c8fe83a-34ff-44f6-80e5-e5bf7880e290", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0c8fe83a-34ff-44f6-80e5-e5bf7880e290") },
		{ "0b8cf87d-78e3-4c75-9f8f-075571e83bce", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0b8cf87d-78e3-4c75-9f8f-075571e83bce") },
		{ "cb29a8e8-f913-4340-b41e-f64c539267b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "cb29a8e8-f913-4340-b41e-f64c539267b0") },
		{ "85e7e371-cbb3-4d90-aeb1-0e3e3c879ce0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "98263477-27f9-44a7-a708-3203c0d5319c"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "85e7e371-cbb3-4d90-aeb1-0e3e3c879ce0") },
		{ "c8c09e44-9689-4e17-a45f-d2479165b0d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "c8c09e44-9689-4e17-a45f-d2479165b0d0") },
		{ "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "3b2d1116-a526-446a-84b9-4e201bee85f2"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "f5f86e69-f2ba-4d6e-89f6-6ed01a7e785c"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c") },
		{ "f37e5e71-a874-42d7-a89e-3e84a834a262", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "8a192745-a8a6-426a-8612-92e78beda675"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f37e5e71-a874-42d7-a89e-3e84a834a262") },
		{ "25602278-82f6-44fe-b07b-29c2e2980630", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "25602278-82f6-44fe-b07b-29c2e2980630") },
		{ "575cc8d6-a444-4bd3-a19b-fcaebf49abae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "575cc8d6-a444-4bd3-a19b-fcaebf49abae") },
		{ "17c015c2-df70-4687-a339-5f3814f0b949", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "17c015c2-df70-4687-a339-5f3814f0b949") },
		{ "43bdb409-df12-4668-897e-2ce4b1c7bf34", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "b7ee5e7b-0ef6-4d00-9e3a-f16c122a24bc"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "43bdb409-df12-4668-897e-2ce4b1c7bf34") },
		{ "fc464fa7-cb96-4341-85f1-fc41468c1e4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "164fcaa3-90eb-4672-bfe2-7aa5309e4bd1"}, { new NonTerminator(NonTerminatorType.IdVarPart), "62d98822-e412-4c61-91ef-6811955d1f78"}, { new Terminator(DelimiterType.LeftSquareBracket), "f6b5d426-4680-4eb7-a920-9416e4fb0f07"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "fc464fa7-cb96-4341-85f1-fc41468c1e4c") },
		{ "39095386-9038-4dcc-b6f4-7042fbaf2194", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "4f70570d-fb17-4089-b22c-e253a429aa92"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "39095386-9038-4dcc-b6f4-7042fbaf2194") },
		{ "1fa9a151-e468-4fe1-b3e2-c0282840644e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7f5a2bec-9062-4ab0-b7db-9fd6dabb3cfe"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1fa9a151-e468-4fe1-b3e2-c0282840644e") },
		{ "e129dcf3-7aab-403a-8dae-2f11d720a236", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "81129e0f-0d42-4880-ae52-eb4cda31f3d9"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e129dcf3-7aab-403a-8dae-2f11d720a236") },
		{ "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21") },
		{ "7c46e9b6-1238-402a-974c-808571dfa726", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7c46e9b6-1238-402a-974c-808571dfa726") },
		{ "ef0634e1-0d3f-41f0-bff3-57bf4e2884e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "ef0634e1-0d3f-41f0-bff3-57bf4e2884e6") },
		{ "faad7a30-d814-4fd5-90e8-c086f9a0e648", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.CompoundStatement), "9a78ea77-a497-4811-93cb-953b89acbff6"}, { new Terminator(KeywordType.Begin), "d2d51360-7c78-47bf-9b9c-bd2f8ebcbe36"},}, new Dictionary<Terminator, ReduceInformation>{ }, "faad7a30-d814-4fd5-90e8-c086f9a0e648") },
		{ "03ea86f5-4260-4120-901a-b29f96c9e9f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.VarDeclaration), "0d0cb9c4-d375-4849-8f3e-dd1c10443eaf"}, { Terminator.IdentifierTerminator, "280b6b74-5a4e-4937-b054-e8f1b11b44a9"},}, new Dictionary<Terminator, ReduceInformation>{ }, "03ea86f5-4260-4120-901a-b29f96c9e9f2") },
		{ "93550301-a8ab-4493-8423-46cc83abe0cd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "02599944-c7af-4ba8-b425-3a24ad54e246"},}, new Dictionary<Terminator, ReduceInformation>{ }, "93550301-a8ab-4493-8423-46cc83abe0cd") },
		{ "f038691f-4133-4728-bd63-eedbb65b0a7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "f038691f-4133-4728-bd63-eedbb65b0a7e") },
		{ "f29e8573-114f-49a5-9d6b-6315292319b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "1379f589-cdbc-40af-8d00-35d0cabc32a5"}, { new Terminator(DelimiterType.Semicolon), "c3d72a2d-8b54-494c-a717-851cb19bd26b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f29e8573-114f-49a5-9d6b-6315292319b4") },
		{ "ab3d74e2-19c7-4f7e-8d3c-9826e996d18f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ParameterList))}, }, "ab3d74e2-19c7-4f7e-8d3c-9826e996d18f") },
		{ "a1cc7797-e56e-43e0-b3dd-dcb9ce0116cc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "a1cc7797-e56e-43e0-b3dd-dcb9ce0116cc") },
		{ "7007b3eb-8197-4235-a415-cb0afeb5e530", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Parameter))}, }, "7007b3eb-8197-4235-a415-cb0afeb5e530") },
		{ "a6f74335-800b-4ee9-bd3c-0ce9bcfc3612", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ValueParameter), "f98ee752-f401-4e1b-9e8e-957adc9790aa"}, { Terminator.IdentifierTerminator, "3725ab6a-0044-4165-85f8-852735c5eed3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a6f74335-800b-4ee9-bd3c-0ce9bcfc3612") },
		{ "3725ab6a-0044-4165-85f8-852735c5eed3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "5bc910a4-45f8-4ad9-b963-d37940c5cff1"}, { new Terminator(DelimiterType.Comma), "5c435b00-58fa-424f-8d80-44581d6495d4"}, { new Terminator(DelimiterType.Colon), "f364b470-5153-4353-abce-800b8bb3b8cf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3725ab6a-0044-4165-85f8-852735c5eed3") },
		{ "59527a1a-de12-484e-9461-0b5863383f5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "cd649cdd-8ef7-457e-be13-7f701e3a0f47"}, { new Terminator(KeywordType.Integer), "53fb2178-3ce3-41bf-b111-74899ad57bab"}, { new Terminator(KeywordType.Real), "f0bab41c-04a7-4c56-b111-ef249f8f888c"}, { new Terminator(KeywordType.Boolean), "ce45f10b-d787-4ea7-9a1b-be114506cc7b"}, { new Terminator(KeywordType.Character), "687ec48b-b819-4790-a279-957de0b38494"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59527a1a-de12-484e-9461-0b5863383f5b") },
		{ "c1dde178-9eec-47d8-8180-fc87cc1865dc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "c1dde178-9eec-47d8-8180-fc87cc1865dc") },
		{ "77142180-a1da-40cd-b722-e9d2d01d1fa4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "061214f5-f509-4581-becb-61ff1ca50f83"}, { new Terminator(DelimiterType.Semicolon), "c3d72a2d-8b54-494c-a717-851cb19bd26b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "77142180-a1da-40cd-b722-e9d2d01d1fa4") },
		{ "cf4d7c50-28fd-4021-8a19-ad8fa3f37c67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "71a04de2-97f3-4363-9ad0-1637eb4a5fcf"}, { new Terminator(DelimiterType.Comma), "c2bd4f6a-1204-4250-9472-219492cfb89b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cf4d7c50-28fd-4021-8a19-ad8fa3f37c67") },
		{ "1332454e-b3a5-4813-b6e5-e5f72f7304b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "63584f3e-27ad-4ea9-b9ef-318d82688721"}, { new Terminator(KeywordType.Integer), "569fea77-5df5-4718-a037-ebf833537bf1"}, { new Terminator(KeywordType.Real), "cab00cd6-e738-4376-b34c-43ed1332a950"}, { new Terminator(KeywordType.Boolean), "14f66661-e8a9-45fe-8089-23d0b7358194"}, { new Terminator(KeywordType.Character), "a8d781f8-e7f2-4d98-9b0d-38ae3572acb4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1332454e-b3a5-4813-b6e5-e5f72f7304b5") },
		{ "cba2bd3d-a0e0-4d1d-9cf3-d7e7cc4c608d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.DoubleDots), "a8b5942a-ccf0-47a1-ab95-aa537e18f756"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cba2bd3d-a0e0-4d1d-9cf3-d7e7cc4c608d") },
		{ "53aabf52-fdfd-4fa4-9450-bbf3ef645f23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Period))}, }, "53aabf52-fdfd-4fa4-9450-bbf3ef645f23") },
		{ "2fd865f8-145a-4a09-bc6a-f8e392402031", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "852581dd-146e-47ab-8131-d60341bc30d0"}, { new NonTerminator(NonTerminatorType.Term), "b14b62af-065d-46c9-83a5-5e219aa5601f"}, { new NonTerminator(NonTerminatorType.Factor), "cb214466-4f8c-4be6-8c5a-aee7453b58fc"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2fd865f8-145a-4a09-bc6a-f8e392402031") },
		{ "e31aba3d-0343-4f1d-a8f0-9230001b9231", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "aff24bec-adef-4a43-8c74-8f5841e96af0"}, { new NonTerminator(NonTerminatorType.Factor), "0ac68032-8218-4afe-a892-68fe4b01ea75"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e31aba3d-0343-4f1d-a8f0-9230001b9231") },
		{ "94df7fe4-5160-4cd6-b591-b6aed9f9680b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7e9d877a-b291-44a7-818b-55eee3312aed"}, { Terminator.NumberTerminator, "8372929f-18a1-42eb-9c56-cdf6b496ab7f"}, { new NonTerminator(NonTerminatorType.Variable), "5eb809a9-364a-4c59-b743-12bff178dcb4"}, { new Terminator(DelimiterType.LeftParenthesis), "31c16865-7977-428f-b0d4-ae5d3ddaf083"}, { Terminator.IdentifierTerminator, "7c517b7f-b3b9-449d-b2e2-4029db449b2b"}, { new Terminator(KeywordType.Not), "8cfe4e5d-b0e0-435c-815e-28e5cdfa237b"}, { new Terminator(OperatorType.Minus), "40916387-5e18-4fcb-83c5-5f1c3d466e52"}, { new Terminator(OperatorType.Plus), "7f3f76ea-4668-486a-a787-4241ae9fff0f"}, { new Terminator(KeywordType.True), "d99f0be4-ea5d-4e5e-b704-9b538b366113"}, { new Terminator(KeywordType.False), "a319032c-8e25-4276-a480-396f2aeae3a6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "94df7fe4-5160-4cd6-b591-b6aed9f9680b") },
		{ "01e94f98-621a-4e26-b0a8-6bac22d690e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "96892e49-473e-478e-9fb1-00e9821e5c49"},}, new Dictionary<Terminator, ReduceInformation>{ }, "01e94f98-621a-4e26-b0a8-6bac22d690e0") },
		{ "6f1aca56-c74b-4803-98a4-2a38b4b14c09", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c79358b6-9450-4cc2-8884-c0a980afe48d"}, { new NonTerminator(NonTerminatorType.ExpressionList), "ae02945c-d62c-404c-9543-9f3a6f983471"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6f1aca56-c74b-4803-98a4-2a38b4b14c09") },
		{ "37f959d9-f414-48e7-8128-443906eb3249", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "37f959d9-f414-48e7-8128-443906eb3249") },
		{ "52b6e6a9-452f-4f0e-96b1-6dc918e7e161", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "9b836f90-31a9-4f71-8e88-5eda74534766"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "52b6e6a9-452f-4f0e-96b1-6dc918e7e161") },
		{ "7222e8f0-4384-41a2-833b-2748ce5171c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7222e8f0-4384-41a2-833b-2748ce5171c4") },
		{ "d19d36d4-dbf0-46a2-9284-a3714ae698c0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "d19d36d4-dbf0-46a2-9284-a3714ae698c0") },
		{ "30e8b3a8-57ed-4c58-a634-a666a35838f8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "30e8b3a8-57ed-4c58-a634-a666a35838f8") },
		{ "84cb6a9e-2732-4e1b-8b6d-9c06fb3ca7b4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "133abdc1-d6c9-46d6-9e3b-ca7b3ce3a4fb"}, { new Terminator(KeywordType.Else), "7e653eb8-6970-4f0d-8a9e-eba566ae8058"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "84cb6a9e-2732-4e1b-8b6d-9c06fb3ca7b4") },
		{ "2003d515-90bb-4516-b5f4-2a4cedf0fad3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "f30c5737-c53a-4cb4-9dde-bbfe4d10865a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2003d515-90bb-4516-b5f4-2a4cedf0fad3") },
		{ "39958051-a99c-49c9-8d22-79d2456b0c78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "39958051-a99c-49c9-8d22-79d2456b0c78") },
		{ "f3c84ef3-69cf-4881-822f-d2a6f0a8f160", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Statement))}, }, "f3c84ef3-69cf-4881-822f-d2a6f0a8f160") },
		{ "2436839b-20b6-4147-af90-9c877a35feea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "6c6783c3-aed7-4475-95d5-49e9c8c81e7f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "4dde76cc-98f5-4ad2-b620-4bf0784bdfc2"}, { new NonTerminator(NonTerminatorType.Term), "2231c4a1-166c-4a9d-a192-0a6f61c69e0a"}, { new NonTerminator(NonTerminatorType.Factor), "15ec65fb-8226-4e0b-9eb4-cdcb95aa1bbf"}, { Terminator.NumberTerminator, "6ee62975-5e32-49ab-a007-124b7506c1dc"}, { new NonTerminator(NonTerminatorType.Variable), "bac3ff40-f069-4e49-b93e-49f067a6fffe"}, { new Terminator(DelimiterType.LeftParenthesis), "d33fe735-f418-4cd6-8430-d08c745447a6"}, { Terminator.IdentifierTerminator, "d7196c33-a63c-4a03-b4e4-c03bb0237729"}, { new Terminator(KeywordType.Not), "41932ced-262a-4a2d-8366-fba91da5a371"}, { new Terminator(OperatorType.Minus), "73ba7b97-7970-4d14-864f-82e10cee8622"}, { new Terminator(OperatorType.Plus), "340cb8fb-7fab-4d27-9190-b9d470c24232"}, { new Terminator(KeywordType.True), "a9a49511-4cbd-4f2d-9f8e-d7c14498737d"}, { new Terminator(KeywordType.False), "446ce9df-7ec8-4a19-b6fa-0379ddc65f9d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2436839b-20b6-4147-af90-9c877a35feea") },
		{ "6afb4fc5-5847-4ac1-8489-830a122891f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "21426d69-e56d-4439-83bb-e69dcaffad37"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6afb4fc5-5847-4ac1-8489-830a122891f2") },
		{ "3e60e08c-33fc-4674-961c-18019584f9ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "94f46443-712c-4c41-9e5e-6cf4b560be22"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "605e6610-c7fd-4688-914d-554d4e1f0467"}, { new NonTerminator(NonTerminatorType.Term), "ecf78c42-792d-4d48-8f88-e1cd8e101dd0"}, { new NonTerminator(NonTerminatorType.Factor), "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3e60e08c-33fc-4674-961c-18019584f9ba") },
		{ "24a23c8f-9000-443b-9fd0-cb6464a74efa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdVarPart), "cb2faf98-22c9-41de-9363-062d1cd8b143"}, { new Terminator(DelimiterType.LeftSquareBracket), "1d4b60ea-bd48-4214-88b6-e64152da03fa"}, { new Terminator(DelimiterType.LeftParenthesis), "cad891ad-68c5-4680-bd73-dbb28cb1d111"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "24a23c8f-9000-443b-9fd0-cb6464a74efa") },
		{ "554d0078-799c-4218-9588-712cf3fe3c14", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "7a572a21-cc15-47b2-820a-ce3923e0e0d6"}, { new NonTerminator(NonTerminatorType.Statement), "f77dde55-48b4-4353-a3c3-37677c20bd47"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "554d0078-799c-4218-9588-712cf3fe3c14") },
		{ "6b1864a6-0e6d-4ead-8f5d-5d65a05384cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "a0bbf310-e7e2-42a3-813b-d06c31fdb7c3"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "6b1864a6-0e6d-4ead-8f5d-5d65a05384cb") },
		{ "2b50054b-9d22-4a3f-8301-d4ac1b214b4c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "813f9fbb-0bdb-4a1d-b917-636d300ada96"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2b50054b-9d22-4a3f-8301-d4ac1b214b4c") },
		{ "d8983d6e-a324-42b1-abda-e5e9b48f6c68", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "d8983d6e-a324-42b1-abda-e5e9b48f6c68") },
		{ "cab4c63c-f4c2-42a6-8d55-15ef272d21b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0") },
		{ "7a2443d4-1bf4-47e0-aa00-769764338863", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7a2443d4-1bf4-47e0-aa00-769764338863") },
		{ "23d5b6c0-049a-44cb-810b-a148cf517a6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "85faa6ce-d0e2-4fc6-afe2-f2f763e6a48a"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "23d5b6c0-049a-44cb-810b-a148cf517a6a") },
		{ "64c0f4db-301a-4e69-b2ff-a804bfc7a354", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "88fdcea8-031a-4add-aa35-1245f70dc560"}, { new NonTerminator(NonTerminatorType.IdVarPart), "ac89b2d3-62ab-4724-adf5-eab3759cc1bf"}, { new Terminator(DelimiterType.LeftSquareBracket), "6506a140-e53f-4029-87c7-1364f07abb74"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "64c0f4db-301a-4e69-b2ff-a804bfc7a354") },
		{ "1472348b-7588-4000-892e-1b4c1c2898c5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "bc474ec4-5cf4-4447-88b0-7d7f6c11f118"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1472348b-7588-4000-892e-1b4c1c2898c5") },
		{ "7a0d66f9-433e-45cc-a555-5baffc6e4a77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7d5c5d91-b1f5-4ca8-afef-981d103924ff"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7a0d66f9-433e-45cc-a555-5baffc6e4a77") },
		{ "abce178f-4211-4c54-9300-e03398fc4b96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "abd5dbd2-4931-438f-9b63-1f3f4ccf034f"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "abce178f-4211-4c54-9300-e03398fc4b96") },
		{ "535aa9c8-1930-4f35-8779-ea515e1f8ac6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "535aa9c8-1930-4f35-8779-ea515e1f8ac6") },
		{ "761bbd44-be16-4366-be63-884e0dcee59e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "761bbd44-be16-4366-be63-884e0dcee59e") },
		{ "37371454-69a0-4bb3-b072-1bc567364aec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "fe7976c2-8b96-4a3c-8d9c-d39df033d42f"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "37371454-69a0-4bb3-b072-1bc567364aec") },
		{ "2323e297-0ba6-44ce-a116-44cf128b60da", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "2323e297-0ba6-44ce-a116-44cf128b60da") },
		{ "168bae91-f78b-49ea-8907-9d29c962343f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "168bae91-f78b-49ea-8907-9d29c962343f") },
		{ "22f51367-b481-4782-9757-0a8e8a79af93", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "df00649b-8c0e-4f9d-abf4-0e9c682b9a20"}, { new NonTerminator(NonTerminatorType.Term), "9bf6096a-4090-46e0-9c21-2f91bbbd84a8"}, { new NonTerminator(NonTerminatorType.Factor), "3f40e54a-b21e-48fe-b0b5-d57112cf5d6d"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "22f51367-b481-4782-9757-0a8e8a79af93") },
		{ "8f2909ee-3fc4-4ae4-9d5d-e81b9d5b0989", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "e0b97216-1322-402d-8421-5d4dd2e6c8c7"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8f2909ee-3fc4-4ae4-9d5d-e81b9d5b0989") },
		{ "2601e0f6-f43b-4d6c-8e4d-e91a592d67b5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "52b0cd30-c38e-4604-bf0b-7b440312308b"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2601e0f6-f43b-4d6c-8e4d-e91a592d67b5") },
		{ "d0e6277a-b96b-4f3f-be22-2991c809852b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a6afe019-2353-491f-8e7e-e74ec4f9060a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d0e6277a-b96b-4f3f-be22-2991c809852b") },
		{ "86898790-5c7b-4d4b-895f-adfdc2f5926a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b3131ead-9c46-4c49-a007-20edab8d6b08"}, { new NonTerminator(NonTerminatorType.ExpressionList), "a64e80cd-c766-4a67-88b5-fb4ff74830bb"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "86898790-5c7b-4d4b-895f-adfdc2f5926a") },
		{ "d9fb70ba-3f04-4fd3-a7be-f2d7f64b40fe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "d9fb70ba-3f04-4fd3-a7be-f2d7f64b40fe") },
		{ "a5abf3c5-3044-4350-ba90-8d4b3fce6f52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "15d5123c-3907-4d3e-96bd-8f5a6658a742"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5abf3c5-3044-4350-ba90-8d4b3fce6f52") },
		{ "3d0615ec-68b6-4d5f-98f0-36c7c5b15ac4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3d0615ec-68b6-4d5f-98f0-36c7c5b15ac4") },
		{ "7399ceb2-ea9f-4ea2-812d-b3d42b6466d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7399ceb2-ea9f-4ea2-812d-b3d42b6466d0") },
		{ "f7b22a84-64db-4655-8c2d-e51593725b63", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f7b22a84-64db-4655-8c2d-e51593725b63") },
		{ "867872bf-45ee-4d6b-9769-fd2b57eceba1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "867872bf-45ee-4d6b-9769-fd2b57eceba1") },
		{ "857ad6f5-4e80-4519-a022-19066c1d24cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "46da40c0-48e9-468e-9857-618e5a6760e7"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "857ad6f5-4e80-4519-a022-19066c1d24cf") },
		{ "d6a9be9f-fda9-4c55-a106-80540580a3d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "1c8e1291-f795-4124-a651-17bec23caba2"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d6a9be9f-fda9-4c55-a106-80540580a3d5") },
		{ "a65c3d77-5617-4761-81a9-4c577341a439", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "595db63b-feec-4644-856b-7441b25e1fb1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a65c3d77-5617-4761-81a9-4c577341a439") },
		{ "b87c5446-1019-4031-b302-7abb9764f456", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "44ddaf01-500a-4146-b9e9-32f24062c7d8"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "881b6451-4f23-4cac-a70c-9559269fd0ee"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "b87c5446-1019-4031-b302-7abb9764f456") },
		{ "9f627a23-8645-4cd3-a1e5-813cf0e39f5e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c75f8261-811d-4aa2-b884-5808a8f68ad6"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9f627a23-8645-4cd3-a1e5-813cf0e39f5e") },
		{ "277e1067-8bdd-4e98-b081-283debc883c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "277e1067-8bdd-4e98-b081-283debc883c7") },
		{ "1c953f12-cf14-406e-903e-470824dc97ad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1c953f12-cf14-406e-903e-470824dc97ad") },
		{ "d73b91e5-02cd-42df-a433-79911f49ccd3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "d73b91e5-02cd-42df-a433-79911f49ccd3") },
		{ "48f64dba-33fb-4f85-831c-a4b339f7eaa9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "24606fa9-2b6e-4cff-b785-edc6652bd5ba"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "48f64dba-33fb-4f85-831c-a4b339f7eaa9") },
		{ "80ffb3c9-cb71-4997-919f-6dbec21ef615", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "ad26c00d-4b80-4437-aa7c-f22ea9f55b5e"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c3c756e1-cf99-445d-93a9-b73b491dbd7a"}, { new Terminator(DelimiterType.LeftSquareBracket), "4b9527ed-50ba-4a72-8c93-2ee35bcfda7e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "80ffb3c9-cb71-4997-919f-6dbec21ef615") },
		{ "ba066289-25d2-4d2d-9fa4-cb70f146976f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a2573c94-3e54-44e9-9f8b-b2d0e8ea6453"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ba066289-25d2-4d2d-9fa4-cb70f146976f") },
		{ "da7f037a-28cb-4723-bfae-e492d20e6613", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2aece360-9191-4648-a9d1-8edb8e96884a"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "da7f037a-28cb-4723-bfae-e492d20e6613") },
		{ "8236a0b9-fcba-4ed2-9896-c83039b99c76", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "3fa35bcb-0aa5-452a-9afe-53363fed2b2a"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8236a0b9-fcba-4ed2-9896-c83039b99c76") },
		{ "c5c55305-06ae-4eb6-8dd0-066ab4045fcd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c5c55305-06ae-4eb6-8dd0-066ab4045fcd") },
		{ "7c79955e-3e28-407b-9bef-fad899315e52", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7c79955e-3e28-407b-9bef-fad899315e52") },
		{ "d1977eb6-08aa-4a64-b5ac-749e2dd30a08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Statement))}, }, "d1977eb6-08aa-4a64-b5ac-749e2dd30a08") },
		{ "43fb9243-dc99-4eb1-883f-65c6186471ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "54870ff5-6830-42be-a0a6-ffc6c6501065"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "43fb9243-dc99-4eb1-883f-65c6186471ee") },
		{ "59b0e133-71b4-45a3-83c1-8971fe651bcc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c8c45668-ae93-4583-bb94-b1fc9655afa2"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "59b0e133-71b4-45a3-83c1-8971fe651bcc") },
		{ "604834f5-b217-434f-9367-7bfed56ab9b0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "604834f5-b217-434f-9367-7bfed56ab9b0") },
		{ "a10429b4-24eb-467f-a608-b98aa8db3e67", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "a10429b4-24eb-467f-a608-b98aa8db3e67") },
		{ "fd00f009-580f-4160-ae7b-6beff969944f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fd00f009-580f-4160-ae7b-6beff969944f") },
		{ "2cb1384f-a831-4ccb-8764-eb3365de539f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "5df0661a-202c-4346-8e20-72161d614790"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2cb1384f-a831-4ccb-8764-eb3365de539f") },
		{ "9a0f0758-e53a-4a96-9702-9c4220cbc1f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "a5492f61-0dd8-4336-97c4-0378a918b307"}, { new NonTerminator(NonTerminatorType.IdVarPart), "8ad5d0ad-3561-4a7a-86c0-07f0c6b90297"}, { new Terminator(DelimiterType.LeftSquareBracket), "5c0cbf48-d7c0-4a91-bfd1-fb8dc9b7081b"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0") },
		{ "55680e38-d0da-441a-b95d-23649cea1617", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "6bee4c38-458d-4351-b1e7-134bf67e68ca"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "55680e38-d0da-441a-b95d-23649cea1617") },
		{ "75a3607c-680a-4a2f-9c66-bc05dff3c7f1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1d95d24a-36da-443b-b49e-cc43d6a59fea"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "75a3607c-680a-4a2f-9c66-bc05dff3c7f1") },
		{ "a21176fc-2cc4-42c8-b918-6ace257cad4d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "e9453497-c7e9-476a-b08a-427f2f43a2be"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a21176fc-2cc4-42c8-b918-6ace257cad4d") },
		{ "216bfda7-db41-4a83-9aa9-49047e81ff99", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "216bfda7-db41-4a83-9aa9-49047e81ff99") },
		{ "1a83cf74-2f22-4578-bacb-ac3b3e307f30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1a83cf74-2f22-4578-bacb-ac3b3e307f30") },
		{ "2849ea39-bcb5-4c15-9f21-310c3c1db53e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "1c13de51-4b5f-47e8-86e1-0179943b6887"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2849ea39-bcb5-4c15-9f21-310c3c1db53e") },
		{ "8dbe4e2c-c11c-45c8-b074-53e60be3ea20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "8dbe4e2c-c11c-45c8-b074-53e60be3ea20") },
		{ "080c9bbf-6992-42eb-9283-477c62b29ed8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "080c9bbf-6992-42eb-9283-477c62b29ed8") },
		{ "f6eef27a-a544-438e-9401-ee1d329edfae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "f6eef27a-a544-438e-9401-ee1d329edfae") },
		{ "9ec9e564-451f-42b5-8167-b460d9f017e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e30986e4-12d0-4bec-98f4-ed089c500fc7"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9ec9e564-451f-42b5-8167-b460d9f017e2") },
		{ "0598be14-ca9b-4e4e-a8f0-fa2b4a328206", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "d544a1df-406e-4fba-ac51-7404ad8687de"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0598be14-ca9b-4e4e-a8f0-fa2b4a328206") },
		{ "39ddeae6-dd11-43a1-9fbc-e64452469d23", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(OperatorType.Assign), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "39ddeae6-dd11-43a1-9fbc-e64452469d23") },
		{ "a5ad624f-614c-4666-8acc-9956c94b8095", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8c78b70b-be8a-4e05-95d1-df7e72b0b7d7"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5ad624f-614c-4666-8acc-9956c94b8095") },
		{ "70777a5f-d416-4536-a439-9f6a25b1e3b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "07f04ab8-b4f5-4324-9792-79151f658da8"}, { new NonTerminator(NonTerminatorType.Term), "b52f9ef7-af24-44c2-ae8e-6ddc1b8bdf10"}, { new NonTerminator(NonTerminatorType.Factor), "0587eecb-ddff-42fd-a87e-a2cc54145a64"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "70777a5f-d416-4536-a439-9f6a25b1e3b3") },
		{ "02ee4aa2-e583-4265-a92e-a32ddb443116", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "08a66bf3-d20f-4540-9dcf-0d1ef09bf430"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "02ee4aa2-e583-4265-a92e-a32ddb443116") },
		{ "47ffd406-53c9-4960-9673-b91d67c6b194", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8fa2b1f9-c9d2-46b6-a9cb-640ce21f4078"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "47ffd406-53c9-4960-9673-b91d67c6b194") },
		{ "59618cd7-7879-4b9f-bc63-65bd9c76777b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a5ce048a-d74f-4040-b053-5e2fc82966f0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "59618cd7-7879-4b9f-bc63-65bd9c76777b") },
		{ "9032e67e-801b-40b3-b9cb-11e1520e7541", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "65d89ddb-78df-48ca-9b07-3f744c4f6855"}, { new NonTerminator(NonTerminatorType.ExpressionList), "36a56723-b1cf-4614-8894-164fed30182a"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9032e67e-801b-40b3-b9cb-11e1520e7541") },
		{ "04b72690-477c-4710-93d3-91c4bba6f269", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "04b72690-477c-4710-93d3-91c4bba6f269") },
		{ "6c1237a0-4351-4409-93b8-bc3f71130644", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "7373be0a-d03b-453d-948f-6128a46ba03f"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6c1237a0-4351-4409-93b8-bc3f71130644") },
		{ "c14e7212-7c5d-4721-8bda-06258999dc3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "c14e7212-7c5d-4721-8bda-06258999dc3f") },
		{ "905f6f19-14dd-4f2e-9d43-6a20248a8b88", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "905f6f19-14dd-4f2e-9d43-6a20248a8b88") },
		{ "78dc179a-8471-4472-ae09-e6d7aeaf2a4e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "78dc179a-8471-4472-ae09-e6d7aeaf2a4e") },
		{ "98263477-27f9-44a7-a708-3203c0d5319c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "98263477-27f9-44a7-a708-3203c0d5319c") },
		{ "42adc07d-e305-4673-b588-72af40c98941", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "8676a9cb-9a2e-4de1-aa70-dd7ed3dd12ef"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "42adc07d-e305-4673-b588-72af40c98941") },
		{ "3b2d1116-a526-446a-84b9-4e201bee85f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "1b2f15a7-639f-4f9f-af90-6d5c35391d3d"}, { new NonTerminator(NonTerminatorType.Term), "f6ef1c84-cd0b-4b13-832e-7c032450e234"}, { new NonTerminator(NonTerminatorType.Factor), "0426cda0-e8fd-49e4-8542-5986243370c9"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3b2d1116-a526-446a-84b9-4e201bee85f2") },
		{ "f5f86e69-f2ba-4d6e-89f6-6ed01a7e785c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "d5655891-1b96-4c39-9c4a-b0f709893320"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f5f86e69-f2ba-4d6e-89f6-6ed01a7e785c") },
		{ "8a192745-a8a6-426a-8612-92e78beda675", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "26e97af7-7f48-414a-8fae-645a2f4c54cb"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8a192745-a8a6-426a-8612-92e78beda675") },
		{ "b7ee5e7b-0ef6-4d00-9e3a-f16c122a24bc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "4b77da3c-d037-499b-9b7d-e16185017ab8"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b7ee5e7b-0ef6-4d00-9e3a-f16c122a24bc") },
		{ "164fcaa3-90eb-4672-bfe2-7aa5309e4bd1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "40677224-2ce7-4f1c-a479-9eacf105f1aa"}, { new NonTerminator(NonTerminatorType.ExpressionList), "c1331848-bf9b-4159-b002-bb04bb2399d1"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "164fcaa3-90eb-4672-bfe2-7aa5309e4bd1") },
		{ "62d98822-e412-4c61-91ef-6811955d1f78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "62d98822-e412-4c61-91ef-6811955d1f78") },
		{ "f6b5d426-4680-4eb7-a920-9416e4fb0f07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "e5fb5661-704f-433e-84c8-25ee0f40a46e"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f6b5d426-4680-4eb7-a920-9416e4fb0f07") },
		{ "4f70570d-fb17-4089-b22c-e253a429aa92", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "4f70570d-fb17-4089-b22c-e253a429aa92") },
		{ "7f5a2bec-9062-4ab0-b7db-9fd6dabb3cfe", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7f5a2bec-9062-4ab0-b7db-9fd6dabb3cfe") },
		{ "81129e0f-0d42-4880-ae52-eb4cda31f3d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "81129e0f-0d42-4880-ae52-eb4cda31f3d9") },
		{ "9a78ea77-a497-4811-93cb-953b89acbff6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SubprogramBody))}, }, "9a78ea77-a497-4811-93cb-953b89acbff6") },
		{ "d2d51360-7c78-47bf-9b9c-bd2f8ebcbe36", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.StatementList), "86cdc12b-8fcc-46d7-a147-1f6089017344"}, { new NonTerminator(NonTerminatorType.Statement), "f77dde55-48b4-4353-a3c3-37677c20bd47"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "d2d51360-7c78-47bf-9b9c-bd2f8ebcbe36") },
		{ "0d0cb9c4-d375-4849-8f3e-dd1c10443eaf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.Semicolon), "af4bc1d2-2918-4737-a024-0122c094d829"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0d0cb9c4-d375-4849-8f3e-dd1c10443eaf") },
		{ "02599944-c7af-4ba8-b425-3a24ad54e246", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "09b6a8d2-888d-452e-ad86-0c969667bd9e"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, { new Terminator(KeywordType.Var), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ConstDeclarations))}, }, "02599944-c7af-4ba8-b425-3a24ad54e246") },
		{ "1379f589-cdbc-40af-8d00-35d0cabc32a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "1379f589-cdbc-40af-8d00-35d0cabc32a5") },
		{ "c3d72a2d-8b54-494c-a717-851cb19bd26b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Parameter), "5ab18416-c1e7-438f-bf53-2a997e6a8bd1"}, { new NonTerminator(NonTerminatorType.VarParameter), "a1cc7797-e56e-43e0-b3dd-dcb9ce0116cc"}, { new NonTerminator(NonTerminatorType.ValueParameter), "7007b3eb-8197-4235-a415-cb0afeb5e530"}, { new Terminator(KeywordType.Var), "a6f74335-800b-4ee9-bd3c-0ce9bcfc3612"}, { Terminator.IdentifierTerminator, "3725ab6a-0044-4165-85f8-852735c5eed3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c3d72a2d-8b54-494c-a717-851cb19bd26b") },
		{ "f98ee752-f401-4e1b-9e8e-957adc9790aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.VarParameter))}, }, "f98ee752-f401-4e1b-9e8e-957adc9790aa") },
		{ "5bc910a4-45f8-4ad9-b963-d37940c5cff1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ValueParameter))}, }, "5bc910a4-45f8-4ad9-b963-d37940c5cff1") },
		{ "5c435b00-58fa-424f-8d80-44581d6495d4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "a0f15d90-d9da-4363-9894-f6af765c23ae"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5c435b00-58fa-424f-8d80-44581d6495d4") },
		{ "f364b470-5153-4353-abce-800b8bb3b8cf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Type), "523fba32-d7c3-460f-b45b-198c04135267"}, { new NonTerminator(NonTerminatorType.BasicType), "5601d86d-b88e-4ad8-bcef-da4a4b8d20bf"}, { new Terminator(KeywordType.Array), "ab73c218-e084-4462-9670-542048ecbb25"}, { new Terminator(KeywordType.Integer), "265dc661-0c5d-49bc-83a8-648314bd547f"}, { new Terminator(KeywordType.Real), "6ba8980f-fabc-4a45-b031-00a2552ccf73"}, { new Terminator(KeywordType.Boolean), "9c4bbea1-b9b8-4d1a-890c-e2825b218b61"}, { new Terminator(KeywordType.Character), "55b917ca-2312-4e43-9503-c6ab992ce9e2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f364b470-5153-4353-abce-800b8bb3b8cf") },
		{ "cd649cdd-8ef7-457e-be13-7f701e3a0f47", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.SubprogramHead))}, }, "cd649cdd-8ef7-457e-be13-7f701e3a0f47") },
		{ "061214f5-f509-4581-becb-61ff1ca50f83", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Colon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.FormalParameter))}, }, "061214f5-f509-4581-becb-61ff1ca50f83") },
		{ "71a04de2-97f3-4363-9ad0-1637eb4a5fcf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "50ad3dfd-cc6a-4d16-ae5b-0cd9c508be48"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71a04de2-97f3-4363-9ad0-1637eb4a5fcf") },
		{ "63584f3e-27ad-4ea9-b9ef-318d82688721", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "63584f3e-27ad-4ea9-b9ef-318d82688721") },
		{ "a8b5942a-ccf0-47a1-ab95-aa537e18f756", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.NumberTerminator, "48e172a4-1a69-4a35-ba02-a15a0baf339d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a8b5942a-ccf0-47a1-ab95-aa537e18f756") },
		{ "852581dd-146e-47ab-8131-d60341bc30d0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "df5dd4b7-4314-44a2-b102-16d1dc01bded"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "852581dd-146e-47ab-8131-d60341bc30d0") },
		{ "b14b62af-065d-46c9-83a5-5e219aa5601f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "eccb4f85-79e0-41bf-afe4-90e595d76176"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b14b62af-065d-46c9-83a5-5e219aa5601f") },
		{ "cb214466-4f8c-4be6-8c5a-aee7453b58fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "cb214466-4f8c-4be6-8c5a-aee7453b58fc") },
		{ "2b088aca-1f66-4735-b43b-647ff6a817d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2b088aca-1f66-4735-b43b-647ff6a817d6") },
		{ "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae") },
		{ "d558e501-5024-4375-a3d0-1120f9e59757", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "bb4c62a9-1c43-4c26-8bb0-1156b149c19c"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d558e501-5024-4375-a3d0-1120f9e59757") },
		{ "19aac92d-1326-45bb-9cde-26c15dfeb3f9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "4d97bdb0-dfde-44ef-9a25-1cc23fa4fd0c"}, { new NonTerminator(NonTerminatorType.IdVarPart), "47d70900-934c-404f-9d0a-4743520d8c81"}, { new Terminator(DelimiterType.LeftSquareBracket), "e6e1354e-8432-4ef5-a58c-e4237bdd4616"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "19aac92d-1326-45bb-9cde-26c15dfeb3f9") },
		{ "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fe686f16-bdeb-4ee6-a328-e15f50c68bc8"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8") },
		{ "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "a0eddb69-2886-4987-854a-de06b7a6e856"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a") },
		{ "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8b36a350-28f2-40f0-888e-9abed7ccc1e2"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07") },
		{ "56c6a91a-0bd1-4c5f-b613-af29bc731cf9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "56c6a91a-0bd1-4c5f-b613-af29bc731cf9") },
		{ "0a1f52df-4d6e-431d-8333-cfc432ac9896", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "0a1f52df-4d6e-431d-8333-cfc432ac9896") },
		{ "aff24bec-adef-4a43-8c74-8f5841e96af0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "94df7fe4-5160-4cd6-b591-b6aed9f9680b"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "aff24bec-adef-4a43-8c74-8f5841e96af0") },
		{ "7e9d877a-b291-44a7-818b-55eee3312aed", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "7e9d877a-b291-44a7-818b-55eee3312aed") },
		{ "96892e49-473e-478e-9fb1-00e9821e5c49", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "96892e49-473e-478e-9fb1-00e9821e5c49") },
		{ "c79358b6-9450-4cc2-8884-c0a980afe48d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "c79358b6-9450-4cc2-8884-c0a980afe48d") },
		{ "ae02945c-d62c-404c-9543-9f3a6f983471", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e12a3e72-3a37-4672-af9f-a2c53884eb07"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ae02945c-d62c-404c-9543-9f3a6f983471") },
		{ "9b836f90-31a9-4f71-8e88-5eda74534766", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "2ea9cdff-25d8-4463-8b91-4eac20d98ff0"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9b836f90-31a9-4f71-8e88-5eda74534766") },
		{ "133abdc1-d6c9-46d6-9e3b-ca7b3ce3a4fb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "133abdc1-d6c9-46d6-9e3b-ca7b3ce3a4fb") },
		{ "7e653eb8-6970-4f0d-8a9e-eba566ae8058", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "19353608-236f-4807-8d7b-86fdf2c9843b"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "7e653eb8-6970-4f0d-8a9e-eba566ae8058") },
		{ "f30c5737-c53a-4cb4-9dde-bbfe4d10865a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "d5a238f9-d488-4209-95ee-d3dfb4d13aef"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "23cfab10-ca75-466c-aee2-950f7ce16203"}, { new NonTerminator(NonTerminatorType.Term), "37f255b4-3fd9-4891-8074-5d7752fff099"}, { new NonTerminator(NonTerminatorType.Factor), "08a56a06-7192-4075-aa0d-0002b0bf12d9"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f30c5737-c53a-4cb4-9dde-bbfe4d10865a") },
		{ "6c6783c3-aed7-4475-95d5-49e9c8c81e7f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Then), "6dd332bd-e5c3-40d1-8053-64f82f76b280"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6c6783c3-aed7-4475-95d5-49e9c8c81e7f") },
		{ "21426d69-e56d-4439-83bb-e69dcaffad37", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(OperatorType.Assign), "98a6ab38-73c3-4ab2-ae90-f54cce6d55ec"},}, new Dictionary<Terminator, ReduceInformation>{ }, "21426d69-e56d-4439-83bb-e69dcaffad37") },
		{ "94f46443-712c-4c41-9e5e-6cf4b560be22", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "75262cee-1b93-4762-ad47-e4bbfcf170f5"},}, new Dictionary<Terminator, ReduceInformation>{ }, "94f46443-712c-4c41-9e5e-6cf4b560be22") },
		{ "cad891ad-68c5-4680-bd73-dbb28cb1d111", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "34cf88f4-2679-46d5-a4bd-857506ee0049"}, { new NonTerminator(NonTerminatorType.ExpressionList), "468d555b-d10c-4e85-96c8-6e1dd45aad3e"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "cad891ad-68c5-4680-bd73-dbb28cb1d111") },
		{ "7a572a21-cc15-47b2-820a-ce3923e0e0d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "4d266410-0f1f-4211-98c8-5ad73e7b8805"}, { new Terminator(DelimiterType.Semicolon), "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7a572a21-cc15-47b2-820a-ce3923e0e0d6") },
		{ "a0bbf310-e7e2-42a3-813b-d06c31fdb7c3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "ead7ee3a-2b9e-491b-a5d1-67feb7e51ec5"}, { new NonTerminator(NonTerminatorType.Factor), "d8983d6e-a324-42b1-abda-e5e9b48f6c68"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a0bbf310-e7e2-42a3-813b-d06c31fdb7c3") },
		{ "813f9fbb-0bdb-4a1d-b917-636d300ada96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "8501d23e-8165-44d6-b3c7-c2d3dc8a6f27"}, { Terminator.NumberTerminator, "cab4c63c-f4c2-42a6-8d55-15ef272d21b0"}, { new NonTerminator(NonTerminatorType.Variable), "7a2443d4-1bf4-47e0-aa00-769764338863"}, { new Terminator(DelimiterType.LeftParenthesis), "23d5b6c0-049a-44cb-810b-a148cf517a6a"}, { Terminator.IdentifierTerminator, "64c0f4db-301a-4e69-b2ff-a804bfc7a354"}, { new Terminator(KeywordType.Not), "1472348b-7588-4000-892e-1b4c1c2898c5"}, { new Terminator(OperatorType.Minus), "7a0d66f9-433e-45cc-a555-5baffc6e4a77"}, { new Terminator(OperatorType.Plus), "abce178f-4211-4c54-9300-e03398fc4b96"}, { new Terminator(KeywordType.True), "535aa9c8-1930-4f35-8779-ea515e1f8ac6"}, { new Terminator(KeywordType.False), "761bbd44-be16-4366-be63-884e0dcee59e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "813f9fbb-0bdb-4a1d-b917-636d300ada96") },
		{ "85faa6ce-d0e2-4fc6-afe2-f2f763e6a48a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "9e57ae92-e6fe-4f62-bbbf-01c54cc0f263"},}, new Dictionary<Terminator, ReduceInformation>{ }, "85faa6ce-d0e2-4fc6-afe2-f2f763e6a48a") },
		{ "88fdcea8-031a-4add-aa35-1245f70dc560", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b72bd367-348f-4fbd-9634-b4577f62e43a"}, { new NonTerminator(NonTerminatorType.ExpressionList), "694346f5-b751-484b-87e0-36acf5eb078d"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "88fdcea8-031a-4add-aa35-1245f70dc560") },
		{ "ac89b2d3-62ab-4724-adf5-eab3759cc1bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "ac89b2d3-62ab-4724-adf5-eab3759cc1bf") },
		{ "6506a140-e53f-4029-87c7-1364f07abb74", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "133ff24f-e11f-450b-ac3a-eaa0bf04dee9"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6506a140-e53f-4029-87c7-1364f07abb74") },
		{ "bc474ec4-5cf4-4447-88b0-7d7f6c11f118", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "bc474ec4-5cf4-4447-88b0-7d7f6c11f118") },
		{ "7d5c5d91-b1f5-4ca8-afef-981d103924ff", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7d5c5d91-b1f5-4ca8-afef-981d103924ff") },
		{ "abd5dbd2-4931-438f-9b63-1f3f4ccf034f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "abd5dbd2-4931-438f-9b63-1f3f4ccf034f") },
		{ "df00649b-8c0e-4f9d-abf4-0e9c682b9a20", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "1b8973f4-0863-40a4-b1fd-2f58fd222c7b"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "df00649b-8c0e-4f9d-abf4-0e9c682b9a20") },
		{ "9bf6096a-4090-46e0-9c21-2f91bbbd84a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "51b6b5d7-42c4-4f55-9c55-72d774cb8f5d"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "9bf6096a-4090-46e0-9c21-2f91bbbd84a8") },
		{ "3f40e54a-b21e-48fe-b0b5-d57112cf5d6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "3f40e54a-b21e-48fe-b0b5-d57112cf5d6d") },
		{ "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049") },
		{ "051e59bb-12b5-4040-a8ed-e3d6deabeb5e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "051e59bb-12b5-4040-a8ed-e3d6deabeb5e") },
		{ "acf16c49-de5a-497f-bc11-fefdcf938c88", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "04cd67d7-c346-41e3-aa4b-2a1fd3496108"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "acf16c49-de5a-497f-bc11-fefdcf938c88") },
		{ "25e4d48e-dd03-4e03-9a71-5577abba56e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "f15133f8-d0f2-4a8d-8b79-573f2d450003"}, { new NonTerminator(NonTerminatorType.IdVarPart), "e7233b40-3e97-411c-90b0-c6891f36223f"}, { new Terminator(DelimiterType.LeftSquareBracket), "3b103296-f9a9-4269-a1d0-ddab20c79956"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "25e4d48e-dd03-4e03-9a71-5577abba56e6") },
		{ "4590ba99-bea5-4252-9711-968ce1ba80a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f1027b2f-2acd-4ee0-9faa-07bc33a3e5f0"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4590ba99-bea5-4252-9711-968ce1ba80a8") },
		{ "4518e39d-ab76-43b2-976b-ada9559918e6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "aeaaed95-4631-443a-a285-8afad64fb8f7"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4518e39d-ab76-43b2-976b-ada9559918e6") },
		{ "b6e23807-1de0-44f0-a845-a82038db0c6e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "adbc7cc6-fc2e-43ec-af23-e9425652639a"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b6e23807-1de0-44f0-a845-a82038db0c6e") },
		{ "23140283-9c44-4e07-811e-014f70593d7d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "23140283-9c44-4e07-811e-014f70593d7d") },
		{ "dc7fb2dd-cdc3-4592-8191-c7e2732548d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "dc7fb2dd-cdc3-4592-8191-c7e2732548d1") },
		{ "e0b97216-1322-402d-8421-5d4dd2e6c8c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "2601e0f6-f43b-4d6c-8e4d-e91a592d67b5"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "e0b97216-1322-402d-8421-5d4dd2e6c8c7") },
		{ "52b0cd30-c38e-4604-bf0b-7b440312308b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "52b0cd30-c38e-4604-bf0b-7b440312308b") },
		{ "a6afe019-2353-491f-8e7e-e74ec4f9060a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a6afe019-2353-491f-8e7e-e74ec4f9060a") },
		{ "b3131ead-9c46-4c49-a007-20edab8d6b08", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b3131ead-9c46-4c49-a007-20edab8d6b08") },
		{ "a64e80cd-c766-4a67-88b5-fb4ff74830bb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "a0d16d8b-5179-41f4-aa3c-fc1f3b2a6c0e"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a64e80cd-c766-4a67-88b5-fb4ff74830bb") },
		{ "15d5123c-3907-4d3e-96bd-8f5a6658a742", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "8b0a337b-63c9-4bc0-ab31-dd4bf7624d3a"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "15d5123c-3907-4d3e-96bd-8f5a6658a742") },
		{ "46da40c0-48e9-468e-9857-618e5a6760e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "46da40c0-48e9-468e-9857-618e5a6760e7") },
		{ "1c8e1291-f795-4124-a651-17bec23caba2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "1c8e1291-f795-4124-a651-17bec23caba2") },
		{ "595db63b-feec-4644-856b-7441b25e1fb1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "f9156b36-b76d-4ba8-852e-2303fcb1a88f"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "605e6610-c7fd-4688-914d-554d4e1f0467"}, { new NonTerminator(NonTerminatorType.Term), "ecf78c42-792d-4d48-8f88-e1cd8e101dd0"}, { new NonTerminator(NonTerminatorType.Factor), "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "595db63b-feec-4644-856b-7441b25e1fb1") },
		{ "44ddaf01-500a-4146-b9e9-32f24062c7d8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "39040f7c-2678-4807-b259-e2e4a2786e3c"}, { new NonTerminator(NonTerminatorType.Term), "c480f64e-d5b8-407a-ae50-c81c38b424b1"}, { new NonTerminator(NonTerminatorType.Factor), "0352fdd2-1b67-477e-93fd-cea1c06f45ee"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "44ddaf01-500a-4146-b9e9-32f24062c7d8") },
		{ "881b6451-4f23-4cac-a70c-9559269fd0ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "51305026-06e1-402c-a236-06774d259ded"}, { new NonTerminator(NonTerminatorType.Factor), "277e1067-8bdd-4e98-b081-283debc883c7"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "881b6451-4f23-4cac-a70c-9559269fd0ee") },
		{ "c75f8261-811d-4aa2-b884-5808a8f68ad6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "bcca27d0-b166-492c-9d16-8cdac6d39c51"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c75f8261-811d-4aa2-b884-5808a8f68ad6") },
		{ "24606fa9-2b6e-4cff-b785-edc6652bd5ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "203d6b48-7a57-4687-a023-37eb3ac2d56a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "24606fa9-2b6e-4cff-b785-edc6652bd5ba") },
		{ "ad26c00d-4b80-4437-aa7c-f22ea9f55b5e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8d57c1ee-e400-4b72-a84d-8eb1973b4cb0"}, { new NonTerminator(NonTerminatorType.ExpressionList), "80d23aab-27ea-4238-9d7e-5686a508a435"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ad26c00d-4b80-4437-aa7c-f22ea9f55b5e") },
		{ "c3c756e1-cf99-445d-93a9-b73b491dbd7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c3c756e1-cf99-445d-93a9-b73b491dbd7a") },
		{ "4b9527ed-50ba-4a72-8c93-2ee35bcfda7e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "dcf0d916-2e5a-483f-b927-a15ffc53fe88"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4b9527ed-50ba-4a72-8c93-2ee35bcfda7e") },
		{ "a2573c94-3e54-44e9-9f8b-b2d0e8ea6453", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a2573c94-3e54-44e9-9f8b-b2d0e8ea6453") },
		{ "2aece360-9191-4648-a9d1-8edb8e96884a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2aece360-9191-4648-a9d1-8edb8e96884a") },
		{ "3fa35bcb-0aa5-452a-9afe-53363fed2b2a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "3fa35bcb-0aa5-452a-9afe-53363fed2b2a") },
		{ "54870ff5-6830-42be-a0a6-ffc6c6501065", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "0fc5f635-8aab-4685-a94e-96488d590aad"}, { new NonTerminator(NonTerminatorType.Factor), "604834f5-b217-434f-9367-7bfed56ab9b0"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "54870ff5-6830-42be-a0a6-ffc6c6501065") },
		{ "c8c45668-ae93-4583-bb94-b1fc9655afa2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "b29ae352-a17f-4ebf-bbcd-73f68116c839"}, { Terminator.NumberTerminator, "a10429b4-24eb-467f-a608-b98aa8db3e67"}, { new NonTerminator(NonTerminatorType.Variable), "fd00f009-580f-4160-ae7b-6beff969944f"}, { new Terminator(DelimiterType.LeftParenthesis), "2cb1384f-a831-4ccb-8764-eb3365de539f"}, { Terminator.IdentifierTerminator, "9a0f0758-e53a-4a96-9702-9c4220cbc1f0"}, { new Terminator(KeywordType.Not), "55680e38-d0da-441a-b95d-23649cea1617"}, { new Terminator(OperatorType.Minus), "75a3607c-680a-4a2f-9c66-bc05dff3c7f1"}, { new Terminator(OperatorType.Plus), "a21176fc-2cc4-42c8-b918-6ace257cad4d"}, { new Terminator(KeywordType.True), "216bfda7-db41-4a83-9aa9-49047e81ff99"}, { new Terminator(KeywordType.False), "1a83cf74-2f22-4578-bacb-ac3b3e307f30"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c8c45668-ae93-4583-bb94-b1fc9655afa2") },
		{ "5df0661a-202c-4346-8e20-72161d614790", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "42b23697-4fd1-467b-913c-3d535a1a5e2e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5df0661a-202c-4346-8e20-72161d614790") },
		{ "a5492f61-0dd8-4336-97c4-0378a918b307", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ad1c4ca8-6ff3-4147-a8fa-d904d54f4bae"}, { new NonTerminator(NonTerminatorType.ExpressionList), "45e57db9-ee65-4d19-a55b-dc8459c17f2f"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a5492f61-0dd8-4336-97c4-0378a918b307") },
		{ "8ad5d0ad-3561-4a7a-86c0-07f0c6b90297", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "8ad5d0ad-3561-4a7a-86c0-07f0c6b90297") },
		{ "5c0cbf48-d7c0-4a91-bfd1-fb8dc9b7081b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "166f3c05-4574-4f5b-aa07-4bc34d007f6b"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "5c0cbf48-d7c0-4a91-bfd1-fb8dc9b7081b") },
		{ "6bee4c38-458d-4351-b1e7-134bf67e68ca", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "6bee4c38-458d-4351-b1e7-134bf67e68ca") },
		{ "1d95d24a-36da-443b-b49e-cc43d6a59fea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1d95d24a-36da-443b-b49e-cc43d6a59fea") },
		{ "e9453497-c7e9-476a-b08a-427f2f43a2be", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "e9453497-c7e9-476a-b08a-427f2f43a2be") },
		{ "e30986e4-12d0-4bec-98f4-ed089c500fc7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e30986e4-12d0-4bec-98f4-ed089c500fc7") },
		{ "d544a1df-406e-4fba-ac51-7404ad8687de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "d544a1df-406e-4fba-ac51-7404ad8687de") },
		{ "8c78b70b-be8a-4e05-95d1-df7e72b0b7d7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "8c78b70b-be8a-4e05-95d1-df7e72b0b7d7") },
		{ "07f04ab8-b4f5-4324-9792-79151f658da8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "28ca6f5f-59a1-49b4-aac5-2a4d7b6b6a71"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "07f04ab8-b4f5-4324-9792-79151f658da8") },
		{ "b52f9ef7-af24-44c2-ae8e-6ddc1b8bdf10", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f4bbb2b0-bcb3-4041-a9d4-6375412f9674"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b52f9ef7-af24-44c2-ae8e-6ddc1b8bdf10") },
		{ "0587eecb-ddff-42fd-a87e-a2cc54145a64", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "0587eecb-ddff-42fd-a87e-a2cc54145a64") },
		{ "fc48fd81-e86c-47c7-87c1-2291bc56bb75", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "fc48fd81-e86c-47c7-87c1-2291bc56bb75") },
		{ "f1d71193-f562-4fd1-818b-760ca17c2a78", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f1d71193-f562-4fd1-818b-760ca17c2a78") },
		{ "50d2792d-8a7e-4a8b-9d3d-8f6350311b18", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "a2209bd2-bc23-4da5-aed3-69ec0c5fd293"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "50d2792d-8a7e-4a8b-9d3d-8f6350311b18") },
		{ "50da5232-d1a0-40f8-9fda-34dd046dd610", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "71648cd3-2916-4ca7-9c7f-c7c419d433e7"}, { new NonTerminator(NonTerminatorType.IdVarPart), "f18bcf42-aeab-4132-8dc7-b302cbab30bf"}, { new Terminator(DelimiterType.LeftSquareBracket), "4cbcf837-89dd-44c4-8298-d0863a7a06de"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "50da5232-d1a0-40f8-9fda-34dd046dd610") },
		{ "caf65160-0457-4b3a-8c36-0f078bfe063b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "7d13108c-7b60-44a7-8cc4-b6c898f358c1"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "caf65160-0457-4b3a-8c36-0f078bfe063b") },
		{ "8a34f1bb-d7aa-4935-899e-48f6bf60755f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "768f97d9-bb41-4bb5-9469-444d622921aa"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "8a34f1bb-d7aa-4935-899e-48f6bf60755f") },
		{ "bf8528f5-d90a-4b7a-b797-42cf8c7c329c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "66c211a1-3758-4bf7-9a58-91a33a10258c"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bf8528f5-d90a-4b7a-b797-42cf8c7c329c") },
		{ "aebc570e-5571-42ba-8636-4a904d7ae434", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "aebc570e-5571-42ba-8636-4a904d7ae434") },
		{ "cf28c26f-f1d0-458b-8357-51a559013c5c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cf28c26f-f1d0-458b-8357-51a559013c5c") },
		{ "08a66bf3-d20f-4540-9dcf-0d1ef09bf430", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "47ffd406-53c9-4960-9673-b91d67c6b194"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "08a66bf3-d20f-4540-9dcf-0d1ef09bf430") },
		{ "8fa2b1f9-c9d2-46b6-a9cb-640ce21f4078", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "8fa2b1f9-c9d2-46b6-a9cb-640ce21f4078") },
		{ "a5ce048a-d74f-4040-b053-5e2fc82966f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "a5ce048a-d74f-4040-b053-5e2fc82966f0") },
		{ "65d89ddb-78df-48ca-9b07-3f744c4f6855", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "65d89ddb-78df-48ca-9b07-3f744c4f6855") },
		{ "36a56723-b1cf-4614-8894-164fed30182a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "50fc22f7-78d7-4abd-8fcc-b4e5d5838f58"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "36a56723-b1cf-4614-8894-164fed30182a") },
		{ "7373be0a-d03b-453d-948f-6128a46ba03f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "68ff2701-5df7-4c5e-ab2b-a71b10befc3f"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7373be0a-d03b-453d-948f-6128a46ba03f") },
		{ "8676a9cb-9a2e-4de1-aa70-dd7ed3dd12ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ExpressionList))}, }, "8676a9cb-9a2e-4de1-aa70-dd7ed3dd12ef") },
		{ "1b2f15a7-639f-4f9f-af90-6d5c35391d3d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "b6afc871-9c24-4801-8215-b46bf4f11edc"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "1b2f15a7-639f-4f9f-af90-6d5c35391d3d") },
		{ "f6ef1c84-cd0b-4b13-832e-7c032450e234", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "691df913-fa28-4dd5-89c9-b6713b30a142"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "f6ef1c84-cd0b-4b13-832e-7c032450e234") },
		{ "0426cda0-e8fd-49e4-8542-5986243370c9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "0426cda0-e8fd-49e4-8542-5986243370c9") },
		{ "2790bace-a273-4647-a94f-0889cbf2b024", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "2790bace-a273-4647-a94f-0889cbf2b024") },
		{ "e1ce60b1-d34e-4bbd-9885-33d2953b99d5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "e1ce60b1-d34e-4bbd-9885-33d2953b99d5") },
		{ "88f5f65d-5a3e-4cb2-af69-05f8991b71e0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "0977bb78-5115-48d5-ae5d-8eabe3d1bde7"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "88f5f65d-5a3e-4cb2-af69-05f8991b71e0") },
		{ "276093ee-50ea-4890-9204-dd4c42d5a6d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "f52b0f6d-32aa-4e8a-9231-1fe029da1b7f"}, { new NonTerminator(NonTerminatorType.IdVarPart), "159aa037-31d2-4a4d-a3a8-b4c9d1069ba1"}, { new Terminator(DelimiterType.LeftSquareBracket), "f2da23e2-6fe1-4a5e-907c-bd9d8a127417"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "276093ee-50ea-4890-9204-dd4c42d5a6d9") },
		{ "d36286b9-630e-4f01-a70c-efc769acec3b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9a19e16f-7cf6-4a88-a9bc-3b24f1df5760"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d36286b9-630e-4f01-a70c-efc769acec3b") },
		{ "e8b2609e-d3b0-4df7-8d30-71682012f622", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fd8c55af-a67a-43c4-bee2-76cb7f7e4de0"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e8b2609e-d3b0-4df7-8d30-71682012f622") },
		{ "661f8d06-c6c5-416f-8574-c1d3498383ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "2cb5b4db-5819-426a-a1c1-74800be3b56e"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "661f8d06-c6c5-416f-8574-c1d3498383ae") },
		{ "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6") },
		{ "4adb4190-f200-4444-916a-25f417059cac", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4adb4190-f200-4444-916a-25f417059cac") },
		{ "d5655891-1b96-4c39-9c4a-b0f709893320", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "8a192745-a8a6-426a-8612-92e78beda675"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "d5655891-1b96-4c39-9c4a-b0f709893320") },
		{ "26e97af7-7f48-414a-8fae-645a2f4c54cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "26e97af7-7f48-414a-8fae-645a2f4c54cb") },
		{ "4b77da3c-d037-499b-9b7d-e16185017ab8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "4b77da3c-d037-499b-9b7d-e16185017ab8") },
		{ "40677224-2ce7-4f1c-a479-9eacf105f1aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "40677224-2ce7-4f1c-a479-9eacf105f1aa") },
		{ "c1331848-bf9b-4159-b002-bb04bb2399d1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "33af1da2-bcf0-4810-807a-5f83701ead15"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c1331848-bf9b-4159-b002-bb04bb2399d1") },
		{ "e5fb5661-704f-433e-84c8-25ee0f40a46e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "b3d609c4-af07-4487-8a49-68d1804b5c77"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e5fb5661-704f-433e-84c8-25ee0f40a46e") },
		{ "86cdc12b-8fcc-46d7-a147-1f6089017344", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.End), "76e2bb63-d910-4147-9ffa-eb8b7471d3ae"}, { new Terminator(DelimiterType.Semicolon), "d7c0ed7a-1000-4e5d-9348-1077a94bf3f4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "86cdc12b-8fcc-46d7-a147-1f6089017344") },
		{ "af4bc1d2-2918-4737-a024-0122c094d829", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { Terminator.IdentifierTerminator, "3bf41f0d-a029-47ff-be8d-6c9bed085d4c"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Begin), new ReduceInformation(3, new NonTerminator(NonTerminatorType.VarDeclarations))}, }, "af4bc1d2-2918-4737-a024-0122c094d829") },
		{ "5ab18416-c1e7-438f-bf53-2a997e6a8bd1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ParameterList))}, }, "5ab18416-c1e7-438f-bf53-2a997e6a8bd1") },
		{ "a0f15d90-d9da-4363-9894-f6af765c23ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.IdentifierList), "c7d257fc-4593-4d05-851a-daea5ccf351a"}, { new Terminator(DelimiterType.Comma), "5c435b00-58fa-424f-8d80-44581d6495d4"}, { new Terminator(DelimiterType.Colon), "f364b470-5153-4353-abce-800b8bb3b8cf"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a0f15d90-d9da-4363-9894-f6af765c23ae") },
		{ "523fba32-d7c3-460f-b45b-198c04135267", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "523fba32-d7c3-460f-b45b-198c04135267") },
		{ "5601d86d-b88e-4ad8-bcef-da4a4b8d20bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Type))}, }, "5601d86d-b88e-4ad8-bcef-da4a4b8d20bf") },
		{ "ab73c218-e084-4462-9670-542048ecbb25", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftSquareBracket), "0340ca99-6908-41ba-af37-5caf29bc08c1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ab73c218-e084-4462-9670-542048ecbb25") },
		{ "265dc661-0c5d-49bc-83a8-648314bd547f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "265dc661-0c5d-49bc-83a8-648314bd547f") },
		{ "6ba8980f-fabc-4a45-b031-00a2552ccf73", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "6ba8980f-fabc-4a45-b031-00a2552ccf73") },
		{ "9c4bbea1-b9b8-4d1a-890c-e2825b218b61", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "9c4bbea1-b9b8-4d1a-890c-e2825b218b61") },
		{ "55b917ca-2312-4e43-9503-c6ab992ce9e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.BasicType))}, }, "55b917ca-2312-4e43-9503-c6ab992ce9e2") },
		{ "50ad3dfd-cc6a-4d16-ae5b-0cd9c508be48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "54ba9775-3dd4-4f76-b5af-cdca98475367"}, { new Terminator(KeywordType.Integer), "53fb2178-3ce3-41bf-b111-74899ad57bab"}, { new Terminator(KeywordType.Real), "f0bab41c-04a7-4c56-b111-ef249f8f888c"}, { new Terminator(KeywordType.Boolean), "ce45f10b-d787-4ea7-9a1b-be114506cc7b"}, { new Terminator(KeywordType.Character), "687ec48b-b819-4790-a279-957de0b38494"},}, new Dictionary<Terminator, ReduceInformation>{ }, "50ad3dfd-cc6a-4d16-ae5b-0cd9c508be48") },
		{ "48e172a4-1a69-4a35-ba02-a15a0baf339d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Period))}, }, "48e172a4-1a69-4a35-ba02-a15a0baf339d") },
		{ "df5dd4b7-4314-44a2-b102-16d1dc01bded", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c2888f10-41bf-41a6-a305-d8d17c81b538"}, { new NonTerminator(NonTerminatorType.Factor), "cb214466-4f8c-4be6-8c5a-aee7453b58fc"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "df5dd4b7-4314-44a2-b102-16d1dc01bded") },
		{ "eccb4f85-79e0-41bf-afe4-90e595d76176", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "90a90ab0-cc22-4807-98a0-ebf59a7e5865"}, { Terminator.NumberTerminator, "2b088aca-1f66-4735-b43b-647ff6a817d6"}, { new NonTerminator(NonTerminatorType.Variable), "54dd46eb-09e8-4045-bfb0-72a0ea4d7fae"}, { new Terminator(DelimiterType.LeftParenthesis), "d558e501-5024-4375-a3d0-1120f9e59757"}, { Terminator.IdentifierTerminator, "19aac92d-1326-45bb-9cde-26c15dfeb3f9"}, { new Terminator(KeywordType.Not), "e2e34c26-1bd4-4a5d-b527-d0f0decad4c8"}, { new Terminator(OperatorType.Minus), "a3509be7-9dc3-4f48-8d60-ec0e43bc2a6a"}, { new Terminator(OperatorType.Plus), "f0af15b7-edf7-40bc-a9ab-3e992e2d6e07"}, { new Terminator(KeywordType.True), "56c6a91a-0bd1-4c5f-b613-af29bc731cf9"}, { new Terminator(KeywordType.False), "0a1f52df-4d6e-431d-8333-cfc432ac9896"},}, new Dictionary<Terminator, ReduceInformation>{ }, "eccb4f85-79e0-41bf-afe4-90e595d76176") },
		{ "bb4c62a9-1c43-4c26-8bb0-1156b149c19c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "715e739b-044b-43f5-ba18-c6489c75e0de"},}, new Dictionary<Terminator, ReduceInformation>{ }, "bb4c62a9-1c43-4c26-8bb0-1156b149c19c") },
		{ "4d97bdb0-dfde-44ef-9a25-1cc23fa4fd0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8ae5a25a-9aa0-4804-847e-b4a64d20698e"}, { new NonTerminator(NonTerminatorType.ExpressionList), "a230436e-e6de-4dd9-96c3-2d4e4b5e3ceb"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4d97bdb0-dfde-44ef-9a25-1cc23fa4fd0c") },
		{ "47d70900-934c-404f-9d0a-4743520d8c81", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "47d70900-934c-404f-9d0a-4743520d8c81") },
		{ "e6e1354e-8432-4ef5-a58c-e4237bdd4616", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "90711a52-036d-48c0-aae1-fd14851af91a"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "e6e1354e-8432-4ef5-a58c-e4237bdd4616") },
		{ "fe686f16-bdeb-4ee6-a328-e15f50c68bc8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "fe686f16-bdeb-4ee6-a328-e15f50c68bc8") },
		{ "a0eddb69-2886-4987-854a-de06b7a6e856", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "a0eddb69-2886-4987-854a-de06b7a6e856") },
		{ "8b36a350-28f2-40f0-888e-9abed7ccc1e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "8b36a350-28f2-40f0-888e-9abed7ccc1e2") },
		{ "e12a3e72-3a37-4672-af9f-a2c53884eb07", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e12a3e72-3a37-4672-af9f-a2c53884eb07") },
		{ "2ea9cdff-25d8-4463-8b91-4eac20d98ff0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "2ea9cdff-25d8-4463-8b91-4eac20d98ff0") },
		{ "19353608-236f-4807-8d7b-86fdf2c9843b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "19353608-236f-4807-8d7b-86fdf2c9843b") },
		{ "d5a238f9-d488-4209-95ee-d3dfb4d13aef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Statement))}, }, "d5a238f9-d488-4209-95ee-d3dfb4d13aef") },
		{ "23cfab10-ca75-466c-aee2-950f7ce16203", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.RelationOperator), "029b7fc2-adb0-49db-9f07-9218db2aca70"}, { new Terminator(OperatorType.Equal), "59e0b42f-0f19-4905-bda8-027e4043f4af"}, { new Terminator(OperatorType.NotEqual), "7863edae-4a97-4391-96b2-2e884ec9446a"}, { new Terminator(OperatorType.Less), "16951dba-a455-4455-ae09-e1f402e42c72"}, { new Terminator(OperatorType.LessEqual), "e510cf8d-4628-45cd-aea4-8b27b1e67c4a"}, { new Terminator(OperatorType.Greater), "59406bfc-b725-4683-80e7-445c998f005a"}, { new Terminator(OperatorType.GreaterEqual), "8f8ba332-6060-40ce-9e86-bdb7a7e3f793"}, { new NonTerminator(NonTerminatorType.AddOperator), "3f724d20-1266-4230-a1b3-7c4625f5b672"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Expression))}, }, "23cfab10-ca75-466c-aee2-950f7ce16203") },
		{ "37f255b4-3fd9-4891-8074-5d7752fff099", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "58aab44d-1d41-4aa5-9a15-8c35b2622033"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "37f255b4-3fd9-4891-8074-5d7752fff099") },
		{ "08a56a06-7192-4075-aa0d-0002b0bf12d9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "08a56a06-7192-4075-aa0d-0002b0bf12d9") },
		{ "b8a51109-ebea-4dbc-ad86-c448a2f43c1f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f") },
		{ "086ebd59-c5fb-44a4-9deb-ecfe51467ac9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "086ebd59-c5fb-44a4-9deb-ecfe51467ac9") },
		{ "7e4af2e1-0d6f-41f6-8a07-b627a3fed888", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "60a3504a-b738-4521-92a3-d2a22ce776db"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7e4af2e1-0d6f-41f6-8a07-b627a3fed888") },
		{ "29da417b-7dc6-462f-a476-f960661330ba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "f5c459d7-abce-4eba-b2f3-1a56d8d15e48"}, { new NonTerminator(NonTerminatorType.IdVarPart), "35c479f7-73bb-4c32-93af-c7e1fa42b4e8"}, { new Terminator(DelimiterType.LeftSquareBracket), "6a76e768-851f-4606-8e4f-35a5501f08f6"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "29da417b-7dc6-462f-a476-f960661330ba") },
		{ "30ad608b-ac4f-41fb-a946-4605750613a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "acf6d057-b9e5-4d29-a35a-5d105ad70b0c"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "30ad608b-ac4f-41fb-a946-4605750613a3") },
		{ "2d620968-101e-42c0-849b-634c1150a000", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "1e66f011-0ed0-4c9c-bafd-9d6f548283e9"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2d620968-101e-42c0-849b-634c1150a000") },
		{ "46d07f42-3046-4717-a29c-a5b8949b28b8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "761f8565-7170-47df-a05a-a5068964ec24"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "46d07f42-3046-4717-a29c-a5b8949b28b8") },
		{ "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1") },
		{ "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061") },
		{ "6dd332bd-e5c3-40d1-8053-64f82f76b280", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "5fbc5866-d2c5-4cef-ac4a-71a104b4a21a"}, { new NonTerminator(NonTerminatorType.Variable), "2003d515-90bb-4516-b5f4-2a4cedf0fad3"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "39958051-a99c-49c9-8d22-79d2456b0c78"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "f3c84ef3-69cf-4881-822f-d2a6f0a8f160"}, { new Terminator(KeywordType.If), "2436839b-20b6-4147-af90-9c877a35feea"}, { new Terminator(KeywordType.For), "6afb4fc5-5847-4ac1-8489-830a122891f2"}, { new Terminator(KeywordType.While), "3e60e08c-33fc-4674-961c-18019584f9ba"}, { Terminator.IdentifierTerminator, "24a23c8f-9000-443b-9fd0-cb6464a74efa"}, { new Terminator(KeywordType.Begin), "554d0078-799c-4218-9588-712cf3fe3c14"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "6dd332bd-e5c3-40d1-8053-64f82f76b280") },
		{ "98a6ab38-73c3-4ab2-ae90-f54cce6d55ec", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "4648455d-153c-44b3-bd15-074fd498eb96"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b87c5446-1019-4031-b302-7abb9764f456"}, { new NonTerminator(NonTerminatorType.Term), "9f627a23-8645-4cd3-a1e5-813cf0e39f5e"}, { new NonTerminator(NonTerminatorType.Factor), "277e1067-8bdd-4e98-b081-283debc883c7"}, { Terminator.NumberTerminator, "1c953f12-cf14-406e-903e-470824dc97ad"}, { new NonTerminator(NonTerminatorType.Variable), "d73b91e5-02cd-42df-a433-79911f49ccd3"}, { new Terminator(DelimiterType.LeftParenthesis), "48f64dba-33fb-4f85-831c-a4b339f7eaa9"}, { Terminator.IdentifierTerminator, "80ffb3c9-cb71-4997-919f-6dbec21ef615"}, { new Terminator(KeywordType.Not), "ba066289-25d2-4d2d-9fa4-cb70f146976f"}, { new Terminator(OperatorType.Minus), "da7f037a-28cb-4723-bfae-e492d20e6613"}, { new Terminator(OperatorType.Plus), "8236a0b9-fcba-4ed2-9896-c83039b99c76"}, { new Terminator(KeywordType.True), "c5c55305-06ae-4eb6-8dd0-066ab4045fcd"}, { new Terminator(KeywordType.False), "7c79955e-3e28-407b-9bef-fad899315e52"},}, new Dictionary<Terminator, ReduceInformation>{ }, "98a6ab38-73c3-4ab2-ae90-f54cce6d55ec") },
		{ "75262cee-1b93-4762-ad47-e4bbfcf170f5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "9989a193-d784-447c-8817-d6b932e85941"}, { new NonTerminator(NonTerminatorType.Variable), "2003d515-90bb-4516-b5f4-2a4cedf0fad3"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "39958051-a99c-49c9-8d22-79d2456b0c78"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "f3c84ef3-69cf-4881-822f-d2a6f0a8f160"}, { new Terminator(KeywordType.If), "2436839b-20b6-4147-af90-9c877a35feea"}, { new Terminator(KeywordType.For), "6afb4fc5-5847-4ac1-8489-830a122891f2"}, { new Terminator(KeywordType.While), "3e60e08c-33fc-4674-961c-18019584f9ba"}, { Terminator.IdentifierTerminator, "24a23c8f-9000-443b-9fd0-cb6464a74efa"}, { new Terminator(KeywordType.Begin), "554d0078-799c-4218-9588-712cf3fe3c14"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "75262cee-1b93-4762-ad47-e4bbfcf170f5") },
		{ "34cf88f4-2679-46d5-a4bd-857506ee0049", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "34cf88f4-2679-46d5-a4bd-857506ee0049") },
		{ "468d555b-d10c-4e85-96c8-6e1dd45aad3e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "13b43611-f2fe-45d1-8f49-908253d1977b"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "468d555b-d10c-4e85-96c8-6e1dd45aad3e") },
		{ "4d266410-0f1f-4211-98c8-5ad73e7b8805", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "4d266410-0f1f-4211-98c8-5ad73e7b8805") },
		{ "ead7ee3a-2b9e-491b-a5d1-67feb7e51ec5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "813f9fbb-0bdb-4a1d-b917-636d300ada96"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "ead7ee3a-2b9e-491b-a5d1-67feb7e51ec5") },
		{ "8501d23e-8165-44d6-b3c7-c2d3dc8a6f27", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "8501d23e-8165-44d6-b3c7-c2d3dc8a6f27") },
		{ "9e57ae92-e6fe-4f62-bbbf-01c54cc0f263", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "9e57ae92-e6fe-4f62-bbbf-01c54cc0f263") },
		{ "b72bd367-348f-4fbd-9634-b4577f62e43a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b72bd367-348f-4fbd-9634-b4577f62e43a") },
		{ "694346f5-b751-484b-87e0-36acf5eb078d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "260f7600-8d79-4546-91c3-df6dcbded3ef"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "694346f5-b751-484b-87e0-36acf5eb078d") },
		{ "133ff24f-e11f-450b-ac3a-eaa0bf04dee9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "b2fd7768-9775-45b0-bc1b-17e914048795"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "133ff24f-e11f-450b-ac3a-eaa0bf04dee9") },
		{ "1b8973f4-0863-40a4-b1fd-2f58fd222c7b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "251401db-0a9e-49f7-90fe-efeefb1eb6af"}, { new NonTerminator(NonTerminatorType.Factor), "3f40e54a-b21e-48fe-b0b5-d57112cf5d6d"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1b8973f4-0863-40a4-b1fd-2f58fd222c7b") },
		{ "51b6b5d7-42c4-4f55-9c55-72d774cb8f5d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "c8f8a271-b110-4599-ab9c-80a71d83082b"}, { Terminator.NumberTerminator, "7391be4b-1fb0-46de-bf0f-bbd6f4ac3049"}, { new NonTerminator(NonTerminatorType.Variable), "051e59bb-12b5-4040-a8ed-e3d6deabeb5e"}, { new Terminator(DelimiterType.LeftParenthesis), "acf16c49-de5a-497f-bc11-fefdcf938c88"}, { Terminator.IdentifierTerminator, "25e4d48e-dd03-4e03-9a71-5577abba56e6"}, { new Terminator(KeywordType.Not), "4590ba99-bea5-4252-9711-968ce1ba80a8"}, { new Terminator(OperatorType.Minus), "4518e39d-ab76-43b2-976b-ada9559918e6"}, { new Terminator(OperatorType.Plus), "b6e23807-1de0-44f0-a845-a82038db0c6e"}, { new Terminator(KeywordType.True), "23140283-9c44-4e07-811e-014f70593d7d"}, { new Terminator(KeywordType.False), "dc7fb2dd-cdc3-4592-8191-c7e2732548d1"},}, new Dictionary<Terminator, ReduceInformation>{ }, "51b6b5d7-42c4-4f55-9c55-72d774cb8f5d") },
		{ "04cd67d7-c346-41e3-aa4b-2a1fd3496108", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0e60dcef-4cff-4da7-a1a0-c3b0bbee222e"},}, new Dictionary<Terminator, ReduceInformation>{ }, "04cd67d7-c346-41e3-aa4b-2a1fd3496108") },
		{ "f15133f8-d0f2-4a8d-8b79-573f2d450003", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7f9dd26d-a132-43e8-9617-0c93a1a59236"}, { new NonTerminator(NonTerminatorType.ExpressionList), "c8e76555-d65f-4f12-9859-160727e38cd9"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f15133f8-d0f2-4a8d-8b79-573f2d450003") },
		{ "e7233b40-3e97-411c-90b0-c6891f36223f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "e7233b40-3e97-411c-90b0-c6891f36223f") },
		{ "3b103296-f9a9-4269-a1d0-ddab20c79956", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "42d56e66-54e8-4d71-9141-2091243dcfea"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3b103296-f9a9-4269-a1d0-ddab20c79956") },
		{ "f1027b2f-2acd-4ee0-9faa-07bc33a3e5f0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f1027b2f-2acd-4ee0-9faa-07bc33a3e5f0") },
		{ "aeaaed95-4631-443a-a285-8afad64fb8f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "aeaaed95-4631-443a-a285-8afad64fb8f7") },
		{ "adbc7cc6-fc2e-43ec-af23-e9425652639a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "adbc7cc6-fc2e-43ec-af23-e9425652639a") },
		{ "a0d16d8b-5179-41f4-aa3c-fc1f3b2a6c0e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "a0d16d8b-5179-41f4-aa3c-fc1f3b2a6c0e") },
		{ "8b0a337b-63c9-4bc0-ab31-dd4bf7624d3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "8b0a337b-63c9-4bc0-ab31-dd4bf7624d3a") },
		{ "f9156b36-b76d-4ba8-852e-2303fcb1a88f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "d856b55b-0c07-408f-a84c-76d88c472994"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f9156b36-b76d-4ba8-852e-2303fcb1a88f") },
		{ "39040f7c-2678-4807-b259-e2e4a2786e3c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "00bc983d-365c-4bdd-9a43-b033f6c5eaba"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "39040f7c-2678-4807-b259-e2e4a2786e3c") },
		{ "c480f64e-d5b8-407a-ae50-c81c38b424b1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "73536a18-df1f-4a4f-81fc-07c2fc3b7ee6"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c480f64e-d5b8-407a-ae50-c81c38b424b1") },
		{ "0352fdd2-1b67-477e-93fd-cea1c06f45ee", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "0352fdd2-1b67-477e-93fd-cea1c06f45ee") },
		{ "597502bd-bfc5-413e-aaf9-a5279dc2d81f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "597502bd-bfc5-413e-aaf9-a5279dc2d81f") },
		{ "68ec7b04-b2ca-44d9-9198-8406d7b95ac8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "68ec7b04-b2ca-44d9-9198-8406d7b95ac8") },
		{ "9fcef660-a3cf-4283-8937-042070fc8839", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "7b24867c-58d5-4d57-94b0-21d29de29033"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9fcef660-a3cf-4283-8937-042070fc8839") },
		{ "546a7d7e-9ec6-4d7f-953d-7e90a46b201e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "1eec7470-9757-4b1d-8827-5a7250e8d73d"}, { new NonTerminator(NonTerminatorType.IdVarPart), "c54dc85c-519e-42b0-8d4d-76543d5e280b"}, { new Terminator(DelimiterType.LeftSquareBracket), "a809a163-178c-447b-bf9f-34a6593d5e60"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e") },
		{ "2b6c5fda-1717-49be-9fa5-41a15c762693", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "fa3b82cf-9e44-4484-b58a-a00d8d51a2a7"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "2b6c5fda-1717-49be-9fa5-41a15c762693") },
		{ "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f97048d3-4c30-492c-8527-7974eabd9e4a"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070") },
		{ "fd1875ee-3911-4420-a664-eefe46f836cb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f5dbf124-2372-42b6-a9bd-3fa0497df370"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "fd1875ee-3911-4420-a664-eefe46f836cb") },
		{ "1a2a9fee-915f-4190-ba8a-a76d5c6b189f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "1a2a9fee-915f-4190-ba8a-a76d5c6b189f") },
		{ "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a") },
		{ "51305026-06e1-402c-a236-06774d259ded", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c75f8261-811d-4aa2-b884-5808a8f68ad6"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "51305026-06e1-402c-a236-06774d259ded") },
		{ "bcca27d0-b166-492c-9d16-8cdac6d39c51", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "bcca27d0-b166-492c-9d16-8cdac6d39c51") },
		{ "203d6b48-7a57-4687-a023-37eb3ac2d56a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "203d6b48-7a57-4687-a023-37eb3ac2d56a") },
		{ "8d57c1ee-e400-4b72-a84d-8eb1973b4cb0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "8d57c1ee-e400-4b72-a84d-8eb1973b4cb0") },
		{ "80d23aab-27ea-4238-9d7e-5686a508a435", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "c6e37e87-e769-46da-bd8f-c4879fd81dc5"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80d23aab-27ea-4238-9d7e-5686a508a435") },
		{ "dcf0d916-2e5a-483f-b927-a15ffc53fe88", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "86c64243-6306-4647-bb5a-e73623da4302"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dcf0d916-2e5a-483f-b927-a15ffc53fe88") },
		{ "0fc5f635-8aab-4685-a94e-96488d590aad", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "c8c45668-ae93-4583-bb94-b1fc9655afa2"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "0fc5f635-8aab-4685-a94e-96488d590aad") },
		{ "b29ae352-a17f-4ebf-bbcd-73f68116c839", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "b29ae352-a17f-4ebf-bbcd-73f68116c839") },
		{ "42b23697-4fd1-467b-913c-3d535a1a5e2e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "42b23697-4fd1-467b-913c-3d535a1a5e2e") },
		{ "ad1c4ca8-6ff3-4147-a8fa-d904d54f4bae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "ad1c4ca8-6ff3-4147-a8fa-d904d54f4bae") },
		{ "45e57db9-ee65-4d19-a55b-dc8459c17f2f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ea36d651-365e-4e3a-b7b0-dd85c4c5f39e"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "45e57db9-ee65-4d19-a55b-dc8459c17f2f") },
		{ "166f3c05-4574-4f5b-aa07-4bc34d007f6b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "526b4e31-2df9-4c72-a672-90a94c67a580"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "166f3c05-4574-4f5b-aa07-4bc34d007f6b") },
		{ "28ca6f5f-59a1-49b4-aac5-2a4d7b6b6a71", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "c2063f2c-47b5-4b72-b490-b11a69a3fc40"}, { new NonTerminator(NonTerminatorType.Factor), "0587eecb-ddff-42fd-a87e-a2cc54145a64"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "28ca6f5f-59a1-49b4-aac5-2a4d7b6b6a71") },
		{ "f4bbb2b0-bcb3-4041-a9d4-6375412f9674", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "10362f85-cc9b-4050-9955-a9482ac3fe16"}, { Terminator.NumberTerminator, "fc48fd81-e86c-47c7-87c1-2291bc56bb75"}, { new NonTerminator(NonTerminatorType.Variable), "f1d71193-f562-4fd1-818b-760ca17c2a78"}, { new Terminator(DelimiterType.LeftParenthesis), "50d2792d-8a7e-4a8b-9d3d-8f6350311b18"}, { Terminator.IdentifierTerminator, "50da5232-d1a0-40f8-9fda-34dd046dd610"}, { new Terminator(KeywordType.Not), "caf65160-0457-4b3a-8c36-0f078bfe063b"}, { new Terminator(OperatorType.Minus), "8a34f1bb-d7aa-4935-899e-48f6bf60755f"}, { new Terminator(OperatorType.Plus), "bf8528f5-d90a-4b7a-b797-42cf8c7c329c"}, { new Terminator(KeywordType.True), "aebc570e-5571-42ba-8636-4a904d7ae434"}, { new Terminator(KeywordType.False), "cf28c26f-f1d0-458b-8357-51a559013c5c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f4bbb2b0-bcb3-4041-a9d4-6375412f9674") },
		{ "a2209bd2-bc23-4da5-aed3-69ec0c5fd293", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "b6aa3f74-82b1-4dc8-8c73-eeb0c1c8a0c6"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a2209bd2-bc23-4da5-aed3-69ec0c5fd293") },
		{ "71648cd3-2916-4ca7-9c7f-c7c419d433e7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "8a0f6966-2ea9-4619-b793-2c7d13abd9fc"}, { new NonTerminator(NonTerminatorType.ExpressionList), "80b4ba84-83d9-49ab-a192-686bf6402fab"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "71648cd3-2916-4ca7-9c7f-c7c419d433e7") },
		{ "f18bcf42-aeab-4132-8dc7-b302cbab30bf", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "f18bcf42-aeab-4132-8dc7-b302cbab30bf") },
		{ "4cbcf837-89dd-44c4-8298-d0863a7a06de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "36bf5f53-e314-4234-8816-fca5140f26ab"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4cbcf837-89dd-44c4-8298-d0863a7a06de") },
		{ "7d13108c-7b60-44a7-8cc4-b6c898f358c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "7d13108c-7b60-44a7-8cc4-b6c898f358c1") },
		{ "768f97d9-bb41-4bb5-9469-444d622921aa", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "768f97d9-bb41-4bb5-9469-444d622921aa") },
		{ "66c211a1-3758-4bf7-9a58-91a33a10258c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "66c211a1-3758-4bf7-9a58-91a33a10258c") },
		{ "50fc22f7-78d7-4abd-8fcc-b4e5d5838f58", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "50fc22f7-78d7-4abd-8fcc-b4e5d5838f58") },
		{ "68ff2701-5df7-4c5e-ab2b-a71b10befc3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "68ff2701-5df7-4c5e-ab2b-a71b10befc3f") },
		{ "b6afc871-9c24-4801-8215-b46bf4f11edc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "b9842ec9-debc-4ba0-9cb5-11f59fa2f61b"}, { new NonTerminator(NonTerminatorType.Factor), "0426cda0-e8fd-49e4-8542-5986243370c9"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b6afc871-9c24-4801-8215-b46bf4f11edc") },
		{ "691df913-fa28-4dd5-89c9-b6713b30a142", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d8bed7ea-b306-4242-ab1b-a4ac02a772f7"}, { Terminator.NumberTerminator, "2790bace-a273-4647-a94f-0889cbf2b024"}, { new NonTerminator(NonTerminatorType.Variable), "e1ce60b1-d34e-4bbd-9885-33d2953b99d5"}, { new Terminator(DelimiterType.LeftParenthesis), "88f5f65d-5a3e-4cb2-af69-05f8991b71e0"}, { Terminator.IdentifierTerminator, "276093ee-50ea-4890-9204-dd4c42d5a6d9"}, { new Terminator(KeywordType.Not), "d36286b9-630e-4f01-a70c-efc769acec3b"}, { new Terminator(OperatorType.Minus), "e8b2609e-d3b0-4df7-8d30-71682012f622"}, { new Terminator(OperatorType.Plus), "661f8d06-c6c5-416f-8574-c1d3498383ae"}, { new Terminator(KeywordType.True), "bcb4a4ef-48fa-40a6-a57f-b8d6ce5041a6"}, { new Terminator(KeywordType.False), "4adb4190-f200-4444-916a-25f417059cac"},}, new Dictionary<Terminator, ReduceInformation>{ }, "691df913-fa28-4dd5-89c9-b6713b30a142") },
		{ "0977bb78-5115-48d5-ae5d-8eabe3d1bde7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "90121335-308a-4ebb-8070-00e8a9c4025d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0977bb78-5115-48d5-ae5d-8eabe3d1bde7") },
		{ "f52b0f6d-32aa-4e8a-9231-1fe029da1b7f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "0909b3db-c7df-4de4-83fd-e068696488f2"}, { new NonTerminator(NonTerminatorType.ExpressionList), "4497e8b0-f729-41ef-af61-21921e12b83b"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f52b0f6d-32aa-4e8a-9231-1fe029da1b7f") },
		{ "159aa037-31d2-4a4d-a3a8-b4c9d1069ba1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "159aa037-31d2-4a4d-a3a8-b4c9d1069ba1") },
		{ "f2da23e2-6fe1-4a5e-907c-bd9d8a127417", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "326cba85-78bb-456d-84a8-1106bf2548a5"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f2da23e2-6fe1-4a5e-907c-bd9d8a127417") },
		{ "9a19e16f-7cf6-4a88-a9bc-3b24f1df5760", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9a19e16f-7cf6-4a88-a9bc-3b24f1df5760") },
		{ "fd8c55af-a67a-43c4-bee2-76cb7f7e4de0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "fd8c55af-a67a-43c4-bee2-76cb7f7e4de0") },
		{ "2cb5b4db-5819-426a-a1c1-74800be3b56e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "2cb5b4db-5819-426a-a1c1-74800be3b56e") },
		{ "33af1da2-bcf0-4810-807a-5f83701ead15", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "33af1da2-bcf0-4810-807a-5f83701ead15") },
		{ "b3d609c4-af07-4487-8a49-68d1804b5c77", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b3d609c4-af07-4487-8a49-68d1804b5c77") },
		{ "76e2bb63-d910-4147-9ffa-eb8b7471d3ae", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.CompoundStatement))}, }, "76e2bb63-d910-4147-9ffa-eb8b7471d3ae") },
		{ "c7d257fc-4593-4d05-851a-daea5ccf351a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdentifierList))}, }, "c7d257fc-4593-4d05-851a-daea5ccf351a") },
		{ "0340ca99-6908-41ba-af37-5caf29bc08c1", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Period), "9502da91-abee-4eb2-a639-fd9285d445d6"}, { Terminator.NumberTerminator, "1d59d28d-3de6-4b9b-aec2-b237c4a2e6dc"},}, new Dictionary<Terminator, ReduceInformation>{ }, "0340ca99-6908-41ba-af37-5caf29bc08c1") },
		{ "54ba9775-3dd4-4f76-b5af-cdca98475367", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "54ba9775-3dd4-4f76-b5af-cdca98475367") },
		{ "c2888f10-41bf-41a6-a305-d8d17c81b538", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "eccb4f85-79e0-41bf-afe4-90e595d76176"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c2888f10-41bf-41a6-a305-d8d17c81b538") },
		{ "90a90ab0-cc22-4807-98a0-ebf59a7e5865", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "90a90ab0-cc22-4807-98a0-ebf59a7e5865") },
		{ "715e739b-044b-43f5-ba18-c6489c75e0de", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "715e739b-044b-43f5-ba18-c6489c75e0de") },
		{ "8ae5a25a-9aa0-4804-847e-b4a64d20698e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "8ae5a25a-9aa0-4804-847e-b4a64d20698e") },
		{ "a230436e-e6de-4dd9-96c3-2d4e4b5e3ceb", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e2100de9-9ca4-4186-9c9a-b7f5a5d77673"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a230436e-e6de-4dd9-96c3-2d4e4b5e3ceb") },
		{ "90711a52-036d-48c0-aae1-fd14851af91a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "38bc27cd-19cd-4c08-bd58-afae0bf96d9d"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "90711a52-036d-48c0-aae1-fd14851af91a") },
		{ "029b7fc2-adb0-49db-9f07-9218db2aca70", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.SimpleExpression), "8b476bcb-7283-4c34-bd71-ed45789d45db"}, { new NonTerminator(NonTerminatorType.Term), "52d5de2b-aa99-46df-8024-06c05db9bcf7"}, { new NonTerminator(NonTerminatorType.Factor), "2e1310d6-5dfb-4058-a6bd-b7050a3b5e8d"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "029b7fc2-adb0-49db-9f07-9218db2aca70") },
		{ "3f724d20-1266-4230-a1b3-7c4625f5b672", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2fc873e0-76f7-4015-8e3e-9f1226db768d"}, { new NonTerminator(NonTerminatorType.Factor), "08a56a06-7192-4075-aa0d-0002b0bf12d9"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "3f724d20-1266-4230-a1b3-7c4625f5b672") },
		{ "58aab44d-1d41-4aa5-9a15-8c35b2622033", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "f6cc426b-89ec-48b3-acf2-55523bfbbcd9"}, { Terminator.NumberTerminator, "b8a51109-ebea-4dbc-ad86-c448a2f43c1f"}, { new NonTerminator(NonTerminatorType.Variable), "086ebd59-c5fb-44a4-9deb-ecfe51467ac9"}, { new Terminator(DelimiterType.LeftParenthesis), "7e4af2e1-0d6f-41f6-8a07-b627a3fed888"}, { Terminator.IdentifierTerminator, "29da417b-7dc6-462f-a476-f960661330ba"}, { new Terminator(KeywordType.Not), "30ad608b-ac4f-41fb-a946-4605750613a3"}, { new Terminator(OperatorType.Minus), "2d620968-101e-42c0-849b-634c1150a000"}, { new Terminator(OperatorType.Plus), "46d07f42-3046-4717-a29c-a5b8949b28b8"}, { new Terminator(KeywordType.True), "cb47dfe5-9c4c-4202-a2da-fb81ccd838a1"}, { new Terminator(KeywordType.False), "b8063f1a-fd1a-4af1-b8d2-76cbdbd15061"},}, new Dictionary<Terminator, ReduceInformation>{ }, "58aab44d-1d41-4aa5-9a15-8c35b2622033") },
		{ "60a3504a-b738-4521-92a3-d2a22ce776db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "34733c80-aa3e-47e6-8f76-0a02deaaaf9c"},}, new Dictionary<Terminator, ReduceInformation>{ }, "60a3504a-b738-4521-92a3-d2a22ce776db") },
		{ "f5c459d7-abce-4eba-b2f3-1a56d8d15e48", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "250e44c9-37dc-4237-8594-aa23ee8eb3bd"}, { new NonTerminator(NonTerminatorType.ExpressionList), "7da2d262-d876-4760-aa45-e47bf064f843"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f5c459d7-abce-4eba-b2f3-1a56d8d15e48") },
		{ "35c479f7-73bb-4c32-93af-c7e1fa42b4e8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "35c479f7-73bb-4c32-93af-c7e1fa42b4e8") },
		{ "6a76e768-851f-4606-8e4f-35a5501f08f6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "d9d4e5d7-76ea-4fca-a245-0d073b49e48a"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6a76e768-851f-4606-8e4f-35a5501f08f6") },
		{ "acf6d057-b9e5-4d29-a35a-5d105ad70b0c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "acf6d057-b9e5-4d29-a35a-5d105ad70b0c") },
		{ "1e66f011-0ed0-4c9c-bafd-9d6f548283e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "1e66f011-0ed0-4c9c-bafd-9d6f548283e9") },
		{ "761f8565-7170-47df-a05a-a5068964ec24", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "761f8565-7170-47df-a05a-a5068964ec24") },
		{ "5fbc5866-d2c5-4cef-ac4a-71a104b4a21a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ElsePart), "d3bf430f-ff31-4729-b234-cde50d75cf96"}, { new Terminator(KeywordType.Else), "b251bbc6-4ecf-4db0-8962-d50dcb947b3f"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.ElsePart))}, }, "5fbc5866-d2c5-4cef-ac4a-71a104b4a21a") },
		{ "4648455d-153c-44b3-bd15-074fd498eb96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.To), "9954d52d-5f63-460e-b5ec-21a1e2c324a0"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4648455d-153c-44b3-bd15-074fd498eb96") },
		{ "9989a193-d784-447c-8817-d6b932e85941", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Statement))}, }, "9989a193-d784-447c-8817-d6b932e85941") },
		{ "13b43611-f2fe-45d1-8f49-908253d1977b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.ProcedureCall))}, }, "13b43611-f2fe-45d1-8f49-908253d1977b") },
		{ "260f7600-8d79-4546-91c3-df6dcbded3ef", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "260f7600-8d79-4546-91c3-df6dcbded3ef") },
		{ "b2fd7768-9775-45b0-bc1b-17e914048795", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Then), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "b2fd7768-9775-45b0-bc1b-17e914048795") },
		{ "251401db-0a9e-49f7-90fe-efeefb1eb6af", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "51b6b5d7-42c4-4f55-9c55-72d774cb8f5d"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "251401db-0a9e-49f7-90fe-efeefb1eb6af") },
		{ "c8f8a271-b110-4599-ab9c-80a71d83082b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "c8f8a271-b110-4599-ab9c-80a71d83082b") },
		{ "0e60dcef-4cff-4da7-a1a0-c3b0bbee222e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0e60dcef-4cff-4da7-a1a0-c3b0bbee222e") },
		{ "7f9dd26d-a132-43e8-9617-0c93a1a59236", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "7f9dd26d-a132-43e8-9617-0c93a1a59236") },
		{ "c8e76555-d65f-4f12-9859-160727e38cd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "622092ce-c7e7-476a-a062-129e3851190f"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "c8e76555-d65f-4f12-9859-160727e38cd9") },
		{ "42d56e66-54e8-4d71-9141-2091243dcfea", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "5b3956d3-c39f-44c3-8f06-21eed237beb8"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "42d56e66-54e8-4d71-9141-2091243dcfea") },
		{ "d856b55b-0c07-408f-a84c-76d88c472994", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "006f2403-9da7-4215-8800-1cf7050d9173"}, { new NonTerminator(NonTerminatorType.Variable), "ae30128e-efd3-4103-9828-59d8da47b869"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "3fa1f8f1-0592-440a-a7d7-a66f7aaec0d5"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "cc93b19c-954a-4e6d-9ffe-b1d796b56b27"}, { new Terminator(KeywordType.If), "bd6a5146-d6d4-453b-9442-d2d146452e94"}, { new Terminator(KeywordType.For), "138ba7eb-e0e3-403c-a842-2268411e11cc"}, { new Terminator(KeywordType.While), "7e2dfbed-0d91-495d-bb54-0d2c5fcc59ad"}, { Terminator.IdentifierTerminator, "b128ff00-00cc-46f4-b962-7e7174f1fb28"}, { new Terminator(KeywordType.Begin), "ac1bf4c3-ab39-4ece-a684-e40886b7c49d"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "d856b55b-0c07-408f-a84c-76d88c472994") },
		{ "00bc983d-365c-4bdd-9a43-b033f6c5eaba", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "2ce1d5bb-6065-4a80-8283-adda737afd5f"}, { new NonTerminator(NonTerminatorType.Factor), "0352fdd2-1b67-477e-93fd-cea1c06f45ee"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "00bc983d-365c-4bdd-9a43-b033f6c5eaba") },
		{ "73536a18-df1f-4a4f-81fc-07c2fc3b7ee6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "13210c94-53e5-4caf-a917-db8eadf0dc12"}, { Terminator.NumberTerminator, "597502bd-bfc5-413e-aaf9-a5279dc2d81f"}, { new NonTerminator(NonTerminatorType.Variable), "68ec7b04-b2ca-44d9-9198-8406d7b95ac8"}, { new Terminator(DelimiterType.LeftParenthesis), "9fcef660-a3cf-4283-8937-042070fc8839"}, { Terminator.IdentifierTerminator, "546a7d7e-9ec6-4d7f-953d-7e90a46b201e"}, { new Terminator(KeywordType.Not), "2b6c5fda-1717-49be-9fa5-41a15c762693"}, { new Terminator(OperatorType.Minus), "00a34300-5ffd-4bdd-9ad4-dbcd7e51d070"}, { new Terminator(OperatorType.Plus), "fd1875ee-3911-4420-a664-eefe46f836cb"}, { new Terminator(KeywordType.True), "1a2a9fee-915f-4190-ba8a-a76d5c6b189f"}, { new Terminator(KeywordType.False), "4d1f706c-e3ac-4c82-aa63-ee5404f45b3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "73536a18-df1f-4a4f-81fc-07c2fc3b7ee6") },
		{ "7b24867c-58d5-4d57-94b0-21d29de29033", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "782bf379-5001-4f59-bc87-3720a9ba8a6d"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7b24867c-58d5-4d57-94b0-21d29de29033") },
		{ "1eec7470-9757-4b1d-8827-5a7250e8d73d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "7ccc8d6d-f793-4845-aa82-0ce24f71cc0a"}, { new NonTerminator(NonTerminatorType.ExpressionList), "565fa308-b899-42c0-980a-6fdc68b189b3"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1eec7470-9757-4b1d-8827-5a7250e8d73d") },
		{ "c54dc85c-519e-42b0-8d4d-76543d5e280b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "c54dc85c-519e-42b0-8d4d-76543d5e280b") },
		{ "a809a163-178c-447b-bf9f-34a6593d5e60", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "031088c2-5441-45d1-9f6c-cd1c73878f7a"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "a809a163-178c-447b-bf9f-34a6593d5e60") },
		{ "fa3b82cf-9e44-4484-b58a-a00d8d51a2a7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "fa3b82cf-9e44-4484-b58a-a00d8d51a2a7") },
		{ "f97048d3-4c30-492c-8527-7974eabd9e4a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f97048d3-4c30-492c-8527-7974eabd9e4a") },
		{ "f5dbf124-2372-42b6-a9bd-3fa0497df370", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "f5dbf124-2372-42b6-a9bd-3fa0497df370") },
		{ "c6e37e87-e769-46da-bd8f-c4879fd81dc5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "c6e37e87-e769-46da-bd8f-c4879fd81dc5") },
		{ "86c64243-6306-4647-bb5a-e73623da4302", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "86c64243-6306-4647-bb5a-e73623da4302") },
		{ "ea36d651-365e-4e3a-b7b0-dd85c4c5f39e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "ea36d651-365e-4e3a-b7b0-dd85c4c5f39e") },
		{ "526b4e31-2df9-4c72-a672-90a94c67a580", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.Do), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "526b4e31-2df9-4c72-a672-90a94c67a580") },
		{ "c2063f2c-47b5-4b72-b490-b11a69a3fc40", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "f4bbb2b0-bcb3-4041-a9d4-6375412f9674"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "c2063f2c-47b5-4b72-b490-b11a69a3fc40") },
		{ "10362f85-cc9b-4050-9955-a9482ac3fe16", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "10362f85-cc9b-4050-9955-a9482ac3fe16") },
		{ "b6aa3f74-82b1-4dc8-8c73-eeb0c1c8a0c6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "b6aa3f74-82b1-4dc8-8c73-eeb0c1c8a0c6") },
		{ "8a0f6966-2ea9-4619-b793-2c7d13abd9fc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "8a0f6966-2ea9-4619-b793-2c7d13abd9fc") },
		{ "80b4ba84-83d9-49ab-a192-686bf6402fab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "5dfd402e-0552-48ff-be50-7242dea29a53"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "80b4ba84-83d9-49ab-a192-686bf6402fab") },
		{ "36bf5f53-e314-4234-8816-fca5140f26ab", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "01dfe155-4110-4983-94ef-816bbb11b2e9"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "36bf5f53-e314-4234-8816-fca5140f26ab") },
		{ "b9842ec9-debc-4ba0-9cb5-11f59fa2f61b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "691df913-fa28-4dd5-89c9-b6713b30a142"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "b9842ec9-debc-4ba0-9cb5-11f59fa2f61b") },
		{ "d8bed7ea-b306-4242-ab1b-a4ac02a772f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "d8bed7ea-b306-4242-ab1b-a4ac02a772f7") },
		{ "90121335-308a-4ebb-8070-00e8a9c4025d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "90121335-308a-4ebb-8070-00e8a9c4025d") },
		{ "0909b3db-c7df-4de4-83fd-e068696488f2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "0909b3db-c7df-4de4-83fd-e068696488f2") },
		{ "4497e8b0-f729-41ef-af61-21921e12b83b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "53cfd34a-5551-45ea-bde3-28ae16f4403f"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4497e8b0-f729-41ef-af61-21921e12b83b") },
		{ "326cba85-78bb-456d-84a8-1106bf2548a5", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "70a7fee5-008b-4712-a9db-c1fe3ebac723"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "326cba85-78bb-456d-84a8-1106bf2548a5") },
		{ "9502da91-abee-4eb2-a639-fd9285d445d6", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "12edc4fc-b686-404a-afbd-c66eb9074f30"}, { new Terminator(DelimiterType.Comma), "c2bd4f6a-1204-4250-9472-219492cfb89b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9502da91-abee-4eb2-a639-fd9285d445d6") },
		{ "e2100de9-9ca4-4186-9c9a-b7f5a5d77673", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e2100de9-9ca4-4186-9c9a-b7f5a5d77673") },
		{ "38bc27cd-19cd-4c08-bd58-afae0bf96d9d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "38bc27cd-19cd-4c08-bd58-afae0bf96d9d") },
		{ "8b476bcb-7283-4c34-bd71-ed45789d45db", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.AddOperator), "1926b1f3-37a9-43db-9ba1-ee96d64685df"}, { new Terminator(OperatorType.Plus), "ff07e7b0-2318-47ab-b5b3-62d38c00a279"}, { new Terminator(OperatorType.Minus), "5df57e00-07dc-4d54-9421-a3d931a3df6f"}, { new Terminator(KeywordType.Or), "81a1e4b9-aff3-4e40-8c7b-890e05350f4a"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Expression))}, }, "8b476bcb-7283-4c34-bd71-ed45789d45db") },
		{ "52d5de2b-aa99-46df-8024-06c05db9bcf7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "21018068-ff37-41c1-8082-b7fc4d74ad6e"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "52d5de2b-aa99-46df-8024-06c05db9bcf7") },
		{ "2e1310d6-5dfb-4058-a6bd-b7050a3b5e8d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Term))}, }, "2e1310d6-5dfb-4058-a6bd-b7050a3b5e8d") },
		{ "46eb1309-679a-446d-b404-979ecdd43f0f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "46eb1309-679a-446d-b404-979ecdd43f0f") },
		{ "705a9171-8181-44cc-aa97-5b7a38f28e64", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "705a9171-8181-44cc-aa97-5b7a38f28e64") },
		{ "4c29438d-d6bf-42ec-85e1-245b63194ef8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "08de5a39-ec72-46d0-8a31-7ae80b287378"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "861ef0f6-2a98-41f3-9a54-3ec7d2ca945c"}, { new NonTerminator(NonTerminatorType.Term), "14e8d50f-f839-47bc-8625-00061a186a05"}, { new NonTerminator(NonTerminatorType.Factor), "a556177c-acc4-4abb-843b-1217bc959c0a"}, { Terminator.NumberTerminator, "3ea3c950-0614-4e77-875b-198db9b5dec1"}, { new NonTerminator(NonTerminatorType.Variable), "25279cd9-10d3-491c-b22b-ab6d219987b1"}, { new Terminator(DelimiterType.LeftParenthesis), "34905681-5c09-48bb-98d9-c7d0f1220b9f"}, { Terminator.IdentifierTerminator, "1afab765-abab-4ee5-8e81-afb4057ed142"}, { new Terminator(KeywordType.Not), "9c2adb6e-9a6a-4732-a2ec-db4f9110891b"}, { new Terminator(OperatorType.Minus), "c9c87e1a-2f2c-453e-978b-d36fb68ca681"}, { new Terminator(OperatorType.Plus), "c9b6ac1b-3387-4e86-a381-f2142b65e4b4"}, { new Terminator(KeywordType.True), "407254a3-f259-4d76-959f-cc762a30e5e7"}, { new Terminator(KeywordType.False), "fbee4454-02b0-44fc-a152-e08a97231e3a"},}, new Dictionary<Terminator, ReduceInformation>{ }, "4c29438d-d6bf-42ec-85e1-245b63194ef8") },
		{ "61516da4-5be0-44bc-8ca3-d7187ff3b18a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.LeftParenthesis), "1a5f1520-58f3-405d-9e94-f69c55481668"}, { new NonTerminator(NonTerminatorType.IdVarPart), "df1a7cfc-59ec-41a5-a4c0-f890edcbb53f"}, { new Terminator(DelimiterType.LeftSquareBracket), "b48f6358-124e-4f72-8fd7-7a82f878fdfc"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "61516da4-5be0-44bc-8ca3-d7187ff3b18a") },
		{ "6afc86e9-61bc-4778-8991-dc784e0f0973", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "9ee505d2-395f-4ff0-ad7c-050947c89ddd"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "6afc86e9-61bc-4778-8991-dc784e0f0973") },
		{ "f03d8722-683f-4ef1-b73a-5178d4eb69a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "d4383430-09ef-4e68-99ab-601e2735e662"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "f03d8722-683f-4ef1-b73a-5178d4eb69a8") },
		{ "b2256f50-1d16-4f7e-a9a8-9f19126afae9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "0d7b2079-b82a-4fd2-add3-edfa0d33d242"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b2256f50-1d16-4f7e-a9a8-9f19126afae9") },
		{ "f67e4745-4976-4fe7-905d-46a90c22b203", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "f67e4745-4976-4fe7-905d-46a90c22b203") },
		{ "c46b6833-8c9c-4a43-a314-31853ff921a3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(1, new NonTerminator(NonTerminatorType.Factor))}, }, "c46b6833-8c9c-4a43-a314-31853ff921a3") },
		{ "2fc873e0-76f7-4015-8e3e-9f1226db768d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "58aab44d-1d41-4aa5-9a15-8c35b2622033"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2fc873e0-76f7-4015-8e3e-9f1226db768d") },
		{ "f6cc426b-89ec-48b3-acf2-55523bfbbcd9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "f6cc426b-89ec-48b3-acf2-55523bfbbcd9") },
		{ "34733c80-aa3e-47e6-8f76-0a02deaaaf9c", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "34733c80-aa3e-47e6-8f76-0a02deaaaf9c") },
		{ "250e44c9-37dc-4237-8594-aa23ee8eb3bd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "250e44c9-37dc-4237-8594-aa23ee8eb3bd") },
		{ "7da2d262-d876-4760-aa45-e47bf064f843", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "e9d5c9ec-08e6-4527-a856-6244c290785d"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "7da2d262-d876-4760-aa45-e47bf064f843") },
		{ "d9d4e5d7-76ea-4fca-a245-0d073b49e48a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "82d53769-e06e-48c7-b447-5f712d18d388"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "d9d4e5d7-76ea-4fca-a245-0d073b49e48a") },
		{ "d3bf430f-ff31-4729-b234-cde50d75cf96", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(5, new NonTerminator(NonTerminatorType.Statement))}, }, "d3bf430f-ff31-4729-b234-cde50d75cf96") },
		{ "b251bbc6-4ecf-4db0-8962-d50dcb947b3f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "c4439f84-1804-4886-aa38-e247e4bd83c7"}, { new NonTerminator(NonTerminatorType.Variable), "2003d515-90bb-4516-b5f4-2a4cedf0fad3"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "39958051-a99c-49c9-8d22-79d2456b0c78"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "f3c84ef3-69cf-4881-822f-d2a6f0a8f160"}, { new Terminator(KeywordType.If), "2436839b-20b6-4147-af90-9c877a35feea"}, { new Terminator(KeywordType.For), "6afb4fc5-5847-4ac1-8489-830a122891f2"}, { new Terminator(KeywordType.While), "3e60e08c-33fc-4674-961c-18019584f9ba"}, { Terminator.IdentifierTerminator, "24a23c8f-9000-443b-9fd0-cb6464a74efa"}, { new Terminator(KeywordType.Begin), "554d0078-799c-4218-9588-712cf3fe3c14"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "b251bbc6-4ecf-4db0-8962-d50dcb947b3f") },
		{ "9954d52d-5f63-460e-b5ec-21a1e2c324a0", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Expression), "feec124f-b66e-42ef-8170-223c0db51a5b"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "605e6610-c7fd-4688-914d-554d4e1f0467"}, { new NonTerminator(NonTerminatorType.Term), "ecf78c42-792d-4d48-8f88-e1cd8e101dd0"}, { new NonTerminator(NonTerminatorType.Factor), "17ce7de7-ab8c-4389-a0a2-d215ebdc96b1"}, { Terminator.NumberTerminator, "2b65415f-fb75-427d-a310-39b44b8db03a"}, { new NonTerminator(NonTerminatorType.Variable), "80bf23b2-58db-4d1a-84d2-bec3e73c0429"}, { new Terminator(DelimiterType.LeftParenthesis), "1cd421b5-16d2-469d-bb62-1a97353ec366"}, { Terminator.IdentifierTerminator, "db36538c-6d08-4417-a589-ca40963ee1c5"}, { new Terminator(KeywordType.Not), "47a918c3-1ad6-4fb3-aa15-e10f85953c5d"}, { new Terminator(OperatorType.Minus), "4a9f4005-1b1b-445e-905a-389a24c88d05"}, { new Terminator(OperatorType.Plus), "d6df8a91-452b-46a5-b35e-18cb2be98261"}, { new Terminator(KeywordType.True), "c9085b14-74b4-4630-8f90-1009eb9f7674"}, { new Terminator(KeywordType.False), "44c59546-5b4e-48b3-a3eb-de499bb1a988"},}, new Dictionary<Terminator, ReduceInformation>{ }, "9954d52d-5f63-460e-b5ec-21a1e2c324a0") },
		{ "622092ce-c7e7-476a-a062-129e3851190f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "622092ce-c7e7-476a-a062-129e3851190f") },
		{ "5b3956d3-c39f-44c3-8f06-21eed237beb8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "5b3956d3-c39f-44c3-8f06-21eed237beb8") },
		{ "006f2403-9da7-4215-8800-1cf7050d9173", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "006f2403-9da7-4215-8800-1cf7050d9173") },
		{ "2ce1d5bb-6065-4a80-8283-adda737afd5f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "73536a18-df1f-4a4f-81fc-07c2fc3b7ee6"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "2ce1d5bb-6065-4a80-8283-adda737afd5f") },
		{ "13210c94-53e5-4caf-a917-db8eadf0dc12", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "13210c94-53e5-4caf-a917-db8eadf0dc12") },
		{ "782bf379-5001-4f59-bc87-3720a9ba8a6d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "782bf379-5001-4f59-bc87-3720a9ba8a6d") },
		{ "7ccc8d6d-f793-4845-aa82-0ce24f71cc0a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "7ccc8d6d-f793-4845-aa82-0ce24f71cc0a") },
		{ "565fa308-b899-42c0-980a-6fdc68b189b3", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "94a2e06b-c671-41a4-af55-c19a73645836"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "565fa308-b899-42c0-980a-6fdc68b189b3") },
		{ "031088c2-5441-45d1-9f6c-cd1c73878f7a", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "f9e870e6-d95b-46e7-b828-862963e7a498"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "031088c2-5441-45d1-9f6c-cd1c73878f7a") },
		{ "5dfd402e-0552-48ff-be50-7242dea29a53", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "5dfd402e-0552-48ff-be50-7242dea29a53") },
		{ "01dfe155-4110-4983-94ef-816bbb11b2e9", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightSquareBracket), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "01dfe155-4110-4983-94ef-816bbb11b2e9") },
		{ "53cfd34a-5551-45ea-bde3-28ae16f4403f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "53cfd34a-5551-45ea-bde3-28ae16f4403f") },
		{ "70a7fee5-008b-4712-a9db-c1fe3ebac723", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Comma), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "70a7fee5-008b-4712-a9db-c1fe3ebac723") },
		{ "12edc4fc-b686-404a-afbd-c66eb9074f30", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Of), "00131584-9785-404b-9878-423997ce06dd"},}, new Dictionary<Terminator, ReduceInformation>{ }, "12edc4fc-b686-404a-afbd-c66eb9074f30") },
		{ "1926b1f3-37a9-43db-9ba1-ee96d64685df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Term), "400c754d-b8ea-4cce-8383-ede49d00cd95"}, { new NonTerminator(NonTerminatorType.Factor), "2e1310d6-5dfb-4058-a6bd-b7050a3b5e8d"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1926b1f3-37a9-43db-9ba1-ee96d64685df") },
		{ "21018068-ff37-41c1-8082-b7fc4d74ad6e", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Factor), "af322388-0fde-412c-a301-214f04ab41e2"}, { Terminator.NumberTerminator, "46eb1309-679a-446d-b404-979ecdd43f0f"}, { new NonTerminator(NonTerminatorType.Variable), "705a9171-8181-44cc-aa97-5b7a38f28e64"}, { new Terminator(DelimiterType.LeftParenthesis), "4c29438d-d6bf-42ec-85e1-245b63194ef8"}, { Terminator.IdentifierTerminator, "61516da4-5be0-44bc-8ca3-d7187ff3b18a"}, { new Terminator(KeywordType.Not), "6afc86e9-61bc-4778-8991-dc784e0f0973"}, { new Terminator(OperatorType.Minus), "f03d8722-683f-4ef1-b73a-5178d4eb69a8"}, { new Terminator(OperatorType.Plus), "b2256f50-1d16-4f7e-a9a8-9f19126afae9"}, { new Terminator(KeywordType.True), "f67e4745-4976-4fe7-905d-46a90c22b203"}, { new Terminator(KeywordType.False), "c46b6833-8c9c-4a43-a314-31853ff921a3"},}, new Dictionary<Terminator, ReduceInformation>{ }, "21018068-ff37-41c1-8082-b7fc4d74ad6e") },
		{ "08de5a39-ec72-46d0-8a31-7ae80b287378", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "ab734109-7307-4d45-b2db-0aac2014431b"},}, new Dictionary<Terminator, ReduceInformation>{ }, "08de5a39-ec72-46d0-8a31-7ae80b287378") },
		{ "1a5f1520-58f3-405d-9e94-f69c55481668", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "6df342dd-d56c-47c1-8c8f-02294997c0fd"}, { new NonTerminator(NonTerminatorType.ExpressionList), "dbf50e34-dd03-4b2a-b8da-a0351318a7df"}, { new NonTerminator(NonTerminatorType.Expression), "c8c09e44-9689-4e17-a45f-d2479165b0d0"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "b79ef3a8-f7a1-4c8a-a045-f85634c8a63c"}, { new NonTerminator(NonTerminatorType.Term), "f37e5e71-a874-42d7-a89e-3e84a834a262"}, { new NonTerminator(NonTerminatorType.Factor), "25602278-82f6-44fe-b07b-29c2e2980630"}, { Terminator.NumberTerminator, "575cc8d6-a444-4bd3-a19b-fcaebf49abae"}, { new NonTerminator(NonTerminatorType.Variable), "17c015c2-df70-4687-a339-5f3814f0b949"}, { new Terminator(DelimiterType.LeftParenthesis), "43bdb409-df12-4668-897e-2ce4b1c7bf34"}, { Terminator.IdentifierTerminator, "fc464fa7-cb96-4341-85f1-fc41468c1e4c"}, { new Terminator(KeywordType.Not), "39095386-9038-4dcc-b6f4-7042fbaf2194"}, { new Terminator(OperatorType.Minus), "1fa9a151-e468-4fe1-b3e2-c0282840644e"}, { new Terminator(OperatorType.Plus), "e129dcf3-7aab-403a-8dae-2f11d720a236"}, { new Terminator(KeywordType.True), "5e64cbe6-42a4-4ad0-a2cb-0213545dfe21"}, { new Terminator(KeywordType.False), "7c46e9b6-1238-402a-974c-808571dfa726"},}, new Dictionary<Terminator, ReduceInformation>{ }, "1a5f1520-58f3-405d-9e94-f69c55481668") },
		{ "df1a7cfc-59ec-41a5-a4c0-f890edcbb53f", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Variable))}, }, "df1a7cfc-59ec-41a5-a4c0-f890edcbb53f") },
		{ "b48f6358-124e-4f72-8fd7-7a82f878fdfc", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.ExpressionList), "ab1f4ffa-164c-4af5-a317-90b2d313d82d"}, { new NonTerminator(NonTerminatorType.Expression), "9a9ac9be-af8b-4d4d-8bcc-ffac3829a5f2"}, { new NonTerminator(NonTerminatorType.SimpleExpression), "2cb1744c-c440-4a4b-96d5-de8bf99406c5"}, { new NonTerminator(NonTerminatorType.Term), "f995f955-70b5-42d4-8abd-c14d8abf1b85"}, { new NonTerminator(NonTerminatorType.Factor), "e5c504d5-3b0f-40f9-8667-5e2cb304f89b"}, { Terminator.NumberTerminator, "d0117e0b-f480-459d-8474-d2167983bc1a"}, { new NonTerminator(NonTerminatorType.Variable), "4b621297-900b-4d2b-a080-eb3b76f9622c"}, { new Terminator(DelimiterType.LeftParenthesis), "e696d593-3e5a-469f-ac63-2f4798ddd4c1"}, { Terminator.IdentifierTerminator, "a7aa1b0d-692d-4711-9b1b-bf0beeff9348"}, { new Terminator(KeywordType.Not), "9972f94a-15f5-4545-b21e-f6b80df89fc4"}, { new Terminator(OperatorType.Minus), "25ee3621-0f2e-400f-a56e-b58c6924b370"}, { new Terminator(OperatorType.Plus), "c0e51999-5060-441b-b2df-88f948fae74d"}, { new Terminator(KeywordType.True), "0c8fe83a-34ff-44f6-80e5-e5bf7880e290"}, { new Terminator(KeywordType.False), "0b8cf87d-78e3-4c75-9f8f-075571e83bce"},}, new Dictionary<Terminator, ReduceInformation>{ }, "b48f6358-124e-4f72-8fd7-7a82f878fdfc") },
		{ "9ee505d2-395f-4ff0-ad7c-050947c89ddd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "9ee505d2-395f-4ff0-ad7c-050947c89ddd") },
		{ "d4383430-09ef-4e68-99ab-601e2735e662", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "d4383430-09ef-4e68-99ab-601e2735e662") },
		{ "0d7b2079-b82a-4fd2-add3-edfa0d33d242", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.Factor))}, }, "0d7b2079-b82a-4fd2-add3-edfa0d33d242") },
		{ "e9d5c9ec-08e6-4527-a856-6244c290785d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Equal), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Less), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Greater), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "e9d5c9ec-08e6-4527-a856-6244c290785d") },
		{ "82d53769-e06e-48c7-b447-5f712d18d388", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Equal), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.NotEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Less), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.LessEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Greater), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.GreaterEqual), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "82d53769-e06e-48c7-b447-5f712d18d388") },
		{ "c4439f84-1804-4886-aa38-e247e4bd83c7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(KeywordType.Else), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(2, new NonTerminator(NonTerminatorType.ElsePart))}, }, "c4439f84-1804-4886-aa38-e247e4bd83c7") },
		{ "feec124f-b66e-42ef-8170-223c0db51a5b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(KeywordType.Do), "a82406a5-b4b5-4f73-94ac-91bd29f960c4"},}, new Dictionary<Terminator, ReduceInformation>{ }, "feec124f-b66e-42ef-8170-223c0db51a5b") },
		{ "94a2e06b-c671-41a4-af55-c19a73645836", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "94a2e06b-c671-41a4-af55-c19a73645836") },
		{ "f9e870e6-d95b-46e7-b828-862963e7a498", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.To), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "f9e870e6-d95b-46e7-b828-862963e7a498") },
		{ "00131584-9785-404b-9878-423997ce06dd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.BasicType), "c0c8671d-c0e6-41a6-9912-80a2788d3833"}, { new Terminator(KeywordType.Integer), "265dc661-0c5d-49bc-83a8-648314bd547f"}, { new Terminator(KeywordType.Real), "6ba8980f-fabc-4a45-b031-00a2552ccf73"}, { new Terminator(KeywordType.Boolean), "9c4bbea1-b9b8-4d1a-890c-e2825b218b61"}, { new Terminator(KeywordType.Character), "55b917ca-2312-4e43-9503-c6ab992ce9e2"},}, new Dictionary<Terminator, ReduceInformation>{ }, "00131584-9785-404b-9878-423997ce06dd") },
		{ "400c754d-b8ea-4cce-8383-ede49d00cd95", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.MultiplyOperator), "21018068-ff37-41c1-8082-b7fc4d74ad6e"}, { new Terminator(OperatorType.Multiply), "f804a71b-b820-43aa-be44-f8a295bde558"}, { new Terminator(OperatorType.Divide), "8b737830-70f6-4bec-97a5-6296481bf768"}, { new Terminator(KeywordType.Divide), "3505d411-753d-4ad5-baa6-5ce42611bdf5"}, { new Terminator(KeywordType.Mod), "12058fc7-adfa-4bfa-8590-10cae139c4ef"}, { new Terminator(KeywordType.And), "43b7afac-75b0-4e50-94f1-52e653362a16"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.SimpleExpression))}, }, "400c754d-b8ea-4cce-8383-ede49d00cd95") },
		{ "af322388-0fde-412c-a301-214f04ab41e2", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Term))}, }, "af322388-0fde-412c-a301-214f04ab41e2") },
		{ "ab734109-7307-4d45-b2db-0aac2014431b", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "ab734109-7307-4d45-b2db-0aac2014431b") },
		{ "6df342dd-d56c-47c1-8c8f-02294997c0fd", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.Factor))}, }, "6df342dd-d56c-47c1-8c8f-02294997c0fd") },
		{ "dbf50e34-dd03-4b2a-b8da-a0351318a7df", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightParenthesis), "adf5839e-bf83-4316-aaa5-626fa1ac6379"}, { new Terminator(DelimiterType.Comma), "42adc07d-e305-4673-b588-72af40c98941"},}, new Dictionary<Terminator, ReduceInformation>{ }, "dbf50e34-dd03-4b2a-b8da-a0351318a7df") },
		{ "ab1f4ffa-164c-4af5-a317-90b2d313d82d", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new Terminator(DelimiterType.RightSquareBracket), "c3e4693e-5291-43de-ac5f-d077f26b45f7"}, { new Terminator(DelimiterType.Comma), "a5ad624f-614c-4666-8acc-9956c94b8095"},}, new Dictionary<Terminator, ReduceInformation>{ }, "ab1f4ffa-164c-4af5-a317-90b2d313d82d") },
		{ "a82406a5-b4b5-4f73-94ac-91bd29f960c4", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{ { new NonTerminator(NonTerminatorType.Statement), "cedb80df-365b-40c0-aa6a-29e82b1a48a8"}, { new NonTerminator(NonTerminatorType.Variable), "2003d515-90bb-4516-b5f4-2a4cedf0fad3"}, { new NonTerminator(NonTerminatorType.ProcedureCall), "39958051-a99c-49c9-8d22-79d2456b0c78"}, { new NonTerminator(NonTerminatorType.CompoundStatement), "f3c84ef3-69cf-4881-822f-d2a6f0a8f160"}, { new Terminator(KeywordType.If), "2436839b-20b6-4147-af90-9c877a35feea"}, { new Terminator(KeywordType.For), "6afb4fc5-5847-4ac1-8489-830a122891f2"}, { new Terminator(KeywordType.While), "3e60e08c-33fc-4674-961c-18019584f9ba"}, { Terminator.IdentifierTerminator, "24a23c8f-9000-443b-9fd0-cb6464a74efa"}, { new Terminator(KeywordType.Begin), "554d0078-799c-4218-9588-712cf3fe3c14"},}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(0, new NonTerminator(NonTerminatorType.Statement))}, }, "a82406a5-b4b5-4f73-94ac-91bd29f960c4") },
		{ "c0c8671d-c0e6-41a6-9912-80a2788d3833", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(DelimiterType.RightParenthesis), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(6, new NonTerminator(NonTerminatorType.Type))}, }, "c0c8671d-c0e6-41a6-9912-80a2788d3833") },
		{ "adf5839e-bf83-4316-aaa5-626fa1ac6379", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Divide), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Mod), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.And), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Plus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(OperatorType.Minus), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Or), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(KeywordType.Else), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(4, new NonTerminator(NonTerminatorType.Factor))}, }, "adf5839e-bf83-4316-aaa5-626fa1ac6379") },
		{ "c3e4693e-5291-43de-ac5f-d077f26b45f7", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Multiply), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Divide), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Mod), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.And), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Plus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(OperatorType.Minus), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Or), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(KeywordType.Else), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(3, new NonTerminator(NonTerminatorType.IdVarPart))}, }, "c3e4693e-5291-43de-ac5f-d077f26b45f7") },
		{ "cedb80df-365b-40c0-aa6a-29e82b1a48a8", new GeneratedTransformer(new Dictionary<TerminatorBase, string>{}, new Dictionary<Terminator, ReduceInformation>{ { new Terminator(KeywordType.End), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(KeywordType.Else), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, { new Terminator(DelimiterType.Semicolon), new ReduceInformation(8, new NonTerminator(NonTerminatorType.Statement))}, }, "cedb80df-365b-40c0-aa6a-29e82b1a48a8") },
	};

    private GeneratedGrammarParser()
    {
        foreach(GeneratedTransformer transformer in s_transformers.Values)
        {
            transformer.ConstructShiftTable(s_transformers);
        }
    }

    private static GeneratedGrammarParser s_instance = new GeneratedGrammarParser();

    public static GeneratedGrammarParser Instance => s_instance;

	public ITransformer BeginTransformer => s_transformers["59d736fd-fe3f-42d7-9c76-7dab621a50ad"];
	public NonTerminator Begin => new NonTerminator(NonTerminatorType.StartNonTerminator);
}
